
import numpy as np
import sympy as sym

def ac_3ph_4w_gfpizv(grid,vsc_data):
    '''
    VSC with 3 phase and 4 wire working in open loop as a grid former.
    
    '''

    params_dict  = grid.dae['params_dict']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']


    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    omega_coi = sym.Symbol('omega_coi',real=True)
    xi_freq = sym.Symbol('xi_freq',real=True)

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
        
    name = vsc_data['bus']

    # inputs
    e_ao_m,e_bo_m,e_co_m,e_no_m = sym.symbols(f'e_ao_m_{name},e_bo_m_{name},e_co_m_{name},e_no_m_{name}', real=True)
    omega_ref,p_c = sym.symbols(f'omega_{name}_ref,p_c_{name}', real=True)
    phi_a = sym.Symbol(f'phi_a_{name}', real=True)
    phi_b = sym.Symbol(f'phi_b_{name}', real=True)
    phi_c = sym.Symbol(f'phi_c_{name}', real=True)
    phi_n = sym.Symbol(f'phi_n_{name}', real=True)
    v_ra = sym.Symbol(f'v_ra_{name}', real=True)
    v_rb = sym.Symbol(f'v_rb_{name}', real=True)
    v_rc = sym.Symbol(f'v_rc_{name}', real=True)
    v_rn = sym.Symbol(f'v_rn_{name}', real=True)    

    # parameters
    S_n,H,K_f,T_f,K_agc,K_delta  = sym.symbols(f'S_n_{name},H_{name},K_f_{name},T_f_{name},K_agc_{name},K_delta_{name}', real=True)
    R_v,X_v = sym.symbols(f'R_v_{name},X_v_{name}', real=True)
    R_s,R_sn,R_ng = sym.symbols(f'R_{name}_s,R_{name}_sn,R_{name}_ng', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_{name}_s,X_{name}_sn,X_{name}_ng', real=True)
    T_e = sym.Symbol(f'T_e_{name}', real=True)
    K_p = sym.Symbol(f"K_p_{name}", real=True)    
    T_p = sym.Symbol(f"T_p_{name}", real=True)   
    T_c = sym.Symbol(f"T_c_{name}", real=True)   
    T_v = sym.Symbol(f"T_v_{name}", real=True)   

    Droop = sym.Symbol(f"Droop_{name}", real=True)   
    
    # dynamical states
    phi = sym.Symbol(f'phi_{name}', real=True)
    xi_p = sym.Symbol(f'xi_p_{name}', real=True)
    p_ef = sym.Symbol(f'p_ef_{name}', real=True)
    De_ao_m,De_bo_m,De_co_m,De_no_m  = sym.symbols(f'De_ao_m_{name},De_bo_m_{name},De_co_m_{name},De_no_m_{name}', real=True)


    omega = sym.Symbol(f'omega_{name}', real=True)
    
    # algebraic states
    #e_an_i,e_bn_i,e_cn_i,e_ng_i = sym.symbols(f'e_{name}_an_i,e_{name}_bn_i,e_{name}_cn_i,e_{name}_ng_i', real=True)
    #v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'v_{name}_a_r,v_{name}_b_r,v_{name}_c_r,v_{name}_n_r,v_{name}_o_r', real=True)
    #v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'v_{name}_a_i,v_{name}_b_i,v_{name}_c_i,v_{name}_n_i,v_{name}_o_i', real=True)
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_o_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)
    v_mn_r,v_mn_i = sym.symbols(f'v_{name}_mn_r,v_{name}_mn_i', real=True)
    v_tao_r,v_tbo_r,v_tco_r = sym.symbols(f'v_tao_r_{name},v_tbo_r_{name},v_tco_r_{name}', real=True)
    v_tao_i,v_tbo_i,v_tco_i = sym.symbols(f'v_tao_i_{name},v_tbo_i_{name},v_tco_i_{name}', real=True)

    omega = sym.Symbol(f'omega_{name}', real=True)
    p_ef  = sym.Symbol(f'p_ef_{name}', real=True)
    p_m = sym.Symbol(f'p_m_{name}', real=True) 
    p_c = sym.Symbol(f'p_c_{name}', real=True) 
    p_cf= sym.Symbol(f'p_cf_{name}', real=True)   

    e_om_r,e_om_i = sym.symbols(f'e_{name}_om_r,e_{name}_om_i', real=True)
    
    Z_va = R_v + 1j*X_v
    Z_vb = R_v + 1j*X_v
    Z_vc = R_v + 1j*X_v

    Z_sa = R_s + 1j*X_s 
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i

    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_og = v_og_r + 1j*v_og_i

    v_tao = v_tao_r + 1j*v_tao_i
    v_tbo = v_tbo_r + 1j*v_tbo_i
    v_tco = v_tco_r + 1j*v_tco_i

      
    e_ao_r = (e_ao_m+De_ao_m)*sym.cos(phi_a + phi) 
    e_ao_i = (e_ao_m+De_ao_m)*sym.sin(phi_a + phi) 
    e_bo_r = (e_bo_m+De_bo_m)*sym.cos(phi_b + phi-2/3*np.pi) 
    e_bo_i = (e_bo_m+De_bo_m)*sym.sin(phi_b + phi-2/3*np.pi) 
    e_co_r = (e_co_m+De_co_m)*sym.cos(phi_c + phi-4/3*np.pi) 
    e_co_i = (e_co_m+De_co_m)*sym.sin(phi_c + phi-4/3*np.pi) 
    e_no_r = (e_no_m+De_no_m)*sym.cos(phi_n + phi) 
    e_no_i = (e_no_m+De_no_m)*sym.sin(phi_n + phi)

    e_ao_cplx = e_ao_r + 1j*e_ao_i
    e_bo_cplx = e_bo_r + 1j*e_bo_i
    e_co_cplx = e_co_r + 1j*e_co_i
    e_no_cplx = e_no_r + 1j*e_no_i

    v_san = v_sa - v_sn
    v_sbn = v_sb - v_sn
    v_scn = v_sc - v_sn


    eq_v_tao_cplx =  e_ao_cplx - i_sa*Z_va - v_tao   # v_sa = v_sag
    eq_v_tbo_cplx =  e_bo_cplx - i_sb*Z_vb - v_tbo
    eq_v_tco_cplx =  e_co_cplx - i_sc*Z_vc - v_tco
    eq_i_sa_cplx = v_og + v_tao - i_sa*Z_sa - v_sa   # v_sa = v_sag
    eq_i_sb_cplx = v_og + v_tbo - i_sb*Z_sb - v_sb
    eq_i_sc_cplx = v_og + v_tco - i_sc*Z_sc - v_sc
    eq_i_sn_cplx = v_og + v_tno - i_sn*Z_sn - v_sn
    eq_v_og_cplx = i_sa + i_sb + i_sc + i_sn + v_og/Z_ng
    #eq_i_sn_cplx = e_ng_cplx - i_sn*Z_sn - v_ng
    #eq_i_ng_cplx = i_ng + i_sa + i_sb + i_sc + i_sn
    #eq_e_ng_cplx  = -e_ng_cplx  + i_ng*Z_ng

    g_list += [sym.re(eq_v_tao_cplx)] 
    g_list += [sym.re(eq_v_tbo_cplx)] 
    g_list += [sym.re(eq_v_tco_cplx)] 
    g_list += [sym.re(eq_i_sa_cplx)] 
    g_list += [sym.re(eq_i_sb_cplx)] 
    g_list += [sym.re(eq_i_sc_cplx)] 
    g_list += [sym.re(eq_i_sn_cplx)] 
    g_list += [sym.re(eq_v_og_cplx)] 
    g_list += [sym.im(eq_v_tao_cplx)]
    g_list += [sym.im(eq_v_tbo_cplx)]
    g_list += [sym.im(eq_v_tco_cplx)]
    g_list += [sym.im(eq_i_sa_cplx)] 
    g_list += [sym.im(eq_i_sb_cplx)] 
    g_list += [sym.im(eq_i_sc_cplx)] 
    g_list += [sym.im(eq_i_sn_cplx)] 
    g_list += [sym.im(eq_v_og_cplx)] 

    y_ini_list += [v_tao_r,v_tbo_r,v_tco_r,i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r]
    y_ini_list += [v_tao_i,v_tbo_i,v_tco_i,i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]
    y_run_list += [v_tao_r,v_tbo_r,v_tco_r,i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r]
    y_run_list += [v_tao_i,v_tbo_i,v_tco_i,i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]

    #y_ini_str = [str(item) for item in y_list]

    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name,ph)
        grid.dae['g'] [idx_r] += -i_s_r
        grid.dae['g'] [idx_i] += -i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        h_dict.update({f'i_vsc_{name}_{ph}_m':i_s_m})

        
    V_1 = 400/np.sqrt(3)
    V_b = V_1
    #    V_1 = 400/np.sqrt(3)*np.exp(1j*np.deg2rad(0))
    # A_1toabc = np.array([1, alpha**2, alpha])
    #V_abc = V_1 * A_1toabc 
    #e_an_r,e_bn_r,e_cn_r = V_abc.real
    #e_an_i,e_bn_i,e_cn_i = V_abc.imag

    u_ini_dict.update({f'e_ao_m_{name}':V_1,f'e_bo_m_{name}':V_1,f'e_co_m_{name}':V_1,f'e_no_m_{name}':0.0})
    u_ini_dict.update({f'v_ra_{name}':0.0,f'v_rb_{name}':0.0,f'v_rc_{name}':0.0,f'v_rn_{name}':0.0})
    u_run_dict.update({f'e_ao_m_{name}':V_1,f'e_bo_m_{name}':V_1,f'e_co_m_{name}':V_1,f'e_no_m_{name}':0.0})
    u_run_dict.update({f'v_ra_{name}':0.0,f'v_rb_{name}':0.0,f'v_rc_{name}':0.0,f'v_rn_{name}':0.0})

    #u_dict.update({f'phi_{name}':0.0})
    u_ini_dict.update({f'phi_a_{name}':0.0})
    u_ini_dict.update({f'phi_b_{name}':0.0})
    u_ini_dict.update({f'phi_c_{name}':0.0})
    u_ini_dict.update({f'phi_n_{name}':0.0})

    u_ini_dict.update({f'p_c_{name}':0.0})
    u_ini_dict.update({f'omega_{name}_ref':1.0})

    u_run_dict.update({f'phi_a_{name}':0.0})
    u_run_dict.update({f'phi_b_{name}':0.0})
    u_run_dict.update({f'phi_c_{name}':0.0})
    u_run_dict.update({f'phi_n_{name}':0.0})

    u_run_dict.update({f'p_c_{name}':0.0})
    u_run_dict.update({f'omega_{name}_ref':1.0})

    #for ph in ['a','b','c','n']:
    #    u_dict.pop(f'i_{name}_{ph}_r')
    #    u_dict.pop(f'i_{name}_{ph}_i')

    params_dict.update({f'X_v_{name}':vsc_data['X_v'],f'R_v_{name}':vsc_data['R_v']})
    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name}':vsc_data['S_n']})
    params_dict.update({f'T_e_{name}':vsc_data['T_e']})
    params_dict.update({f'T_c_{name}':vsc_data['T_c']})
    params_dict.update({f'T_v_{name}':vsc_data['T_v']})
    params_dict.update({f'Droop_{name}':vsc_data['Droop']})
    params_dict.update({f'K_p_{name}':vsc_data['K_p']})
    params_dict.update({f'T_p_{name}':vsc_data['T_p']})
    params_dict.update({f'K_agc_{name}':vsc_data['K_agc']})
    params_dict.update({f'K_delta_{name}':vsc_data['K_delta']})
    
    
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    

    v_szpn = A_a0*v_sabc
    i_szpn = A_a0*i_sabc
    
    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])
    
    p_pos = sym.re(s_pos)

    p_r = K_agc*xi_freq
    epsilon_p = p_m - p_ef

    dphi   = 2*np.pi*50*(omega - omega_coi) - K_delta*phi
    dxi_p = epsilon_p
    dp_ef = 1/T_e*(p_pos/S_n - p_ef)
    dp_cf = 1/T_c*(p_c - p_cf)
   
    dDe_ao_m = 1/T_v*(v_ra - De_ao_m)
    dDe_bo_m = 1/T_v*(v_rb - De_bo_m)
    dDe_co_m = 1/T_v*(v_rc - De_co_m)
    dDe_no_m = 1/T_v*(v_rn - De_no_m)


    # algebraic equations   
    g_omega = -omega + K_p*(epsilon_p + xi_p/T_p) + 1
    g_p_m  = -p_m + p_cf + p_r - 1/Droop*(omega - omega_ref)

    g_list += [g_omega, g_p_m] 
    y_ini_list += [  omega,   p_m] 
    y_run_list += [  omega,   p_m] 
 
    f_list += [dphi,dxi_p,dp_ef,dp_cf,dDe_ao_m,dDe_bo_m,dDe_co_m,dDe_no_m] #,dDe_ao_m,dDe_bo_m,dDe_co_m,dDe_no_m]
    x_list += [ phi, xi_p, p_ef, p_cf, De_ao_m, De_bo_m, De_co_m, De_no_m] #, De_ao_m, De_bo_m, De_co_m, De_no_m]
    
    h_dict.update({f'p_{name}_pos':sym.re(s_pos),f'p_{name}_neg':sym.re(s_neg),f'p_{name}_zer':sym.re(s_zer)})
    h_dict.update({str(e_ao_m):e_ao_m,str(e_bo_m):e_bo_m,str(e_co_m):e_co_m})
    h_dict.update({str(v_ra):v_ra,str(v_rb):v_rb,str(v_rc):v_rc})

    n2a = {0:'a',1:'b',2:'c'}
    for ph in [0,1,2]:
        s = v_sabc[ph]*sym.conjugate(i_sabc[ph])
        p = sym.re(s)
        q = sym.im(s)
        h_dict.update({f'p_vsc_{name}_{n2a[ph]}':p,f'q_vsc_{name}_{n2a[ph]}':q})

    h_dict.update({str(p_c):p_c,str(omega_ref):omega_ref})
    h_dict.update({str(phi):phi})

    grid.dae['xy_0_dict'].update({'omega':1.0})

    HS_coi  = S_n
    omega_coi_i = S_n*omega

    grid.omega_coi_numerator += omega_coi_i 
    grid.omega_coi_denominator += HS_coi

def test_ib():
    '''
    test grid former connected to infinite bus
    '''
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('ac_3ph_4w_gfpizv_ib.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()    
    p_c = 0.5
    model.ini({'p_c_A1':0.5},'xy_0.json')
    
    model.report_u()
    model.report_x()
    model.report_y()
    model.report_z()

    S_n = model.get_value('S_n_A1')
    assert model.get_value('omega_A1')  == pytest.approx(1)
    assert model.get_value('omega_coi') == pytest.approx(1)
    assert model.get_value('p_vsc_A1_a') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_vsc_A1_b') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_vsc_A1_c') == pytest.approx(p_c*S_n/3, rel=0.05)
    assert model.get_value('p_A2') == pytest.approx(-p_c*S_n, rel=0.05)

def test_iso():
    '''
    test isolated grid former feeding a load
    '''
    import numpy as np
    import sympy as sym
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db
    import pytest

    grid = urisi('ac3ph4wgfpi3_iso.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    model = temp.model()
    p_load_ph =  50e3
    q_load_ph = -20e3
    model.ini({'p_load_A2_a':p_load_ph,'q_load_A2_a':q_load_ph,
               'p_load_A2_b':p_load_ph,'q_load_A2_b':q_load_ph,
               'p_load_A2_c':p_load_ph,'q_load_A2_c':q_load_ph,},'xy_0.json')

    assert model.get_value('omega_A1')  == 1.0
    assert model.get_value('omega_coi') == 1.0
    assert model.get_value('p_vsc_A1_a') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('p_vsc_A1_b') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('p_vsc_A1_c') == pytest.approx(p_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_a') == pytest.approx(q_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_b') == pytest.approx(q_load_ph, rel=0.05)
    assert model.get_value('q_vsc_A1_c') == pytest.approx(q_load_ph, rel=0.05)


if __name__ == '__main__':

    #development()
    test_ib()


# -*- coding: utf-8 -*-
"""
Created on Thu Feb 25 23:52:55 2021

@author: jmmau
"""

import numpy as np
import sympy as sym
import json
import os
import hjson
from pydae.bmapu.syns.syns import add_syns
from pydae.bmapu.vscs.vscs import add_vscs
from pydae.bmapu.vsgs.vsgs import add_vsgs
from pydae.bmapu.wecs.wecs import add_wecs
from pydae.bmapu.pvs.pvs import add_pvs
from pydae.bmapu.loads.loads import add_loads
from pydae.bmapu.sources.sources import add_sources
from pydae.bmapu.miscellaneous.miscellaneous import add_miscellaneous
import pydae.build_cffi as db
from pydae.build_v2 import builder

import requests

# todo:
    # S_base can't be modified becuase impedances in element base are computed
    # in S_base only in the build
class bmapu:
    '''
    

    Parameters
    ----------
    data_input : string or dict
        File path to the system data information or dictionary with the information.

    Returns
    -------
    dict
        Dictionary with the equations for pydae. 
        
    {
     'sys':{'name':'pf_1','S_base':100e6},       
     'buses':[{'name':'GRI','P_W':0.0,'Q_var':0.0,'U_kV':66.0, 'type':'slack'},
              {'name':'POI','P_W':0.0,'Q_var':0.0,'U_kV':66.0},
              {'name':'PMV','P_W':0.0,'Q_var':0.0,'U_kV':20.0}],
     'lines':[{'bus_j':'GRI','bus_k':'POI','X_km':0.4,'R_km':0.12,'km':20},
              {'bus_j':'POI','bus_k':'PMV','X_pu':0.04,'R_pu':0.01, 'S_mva':50.0}]
    }
        

    '''

    def __init__(self,data_input='',testing=False):
        
        if type(data_input) == str:
            if 'http' in data_input:
                url = data_input
                resp = requests.get(url)
                data = hjson.loads(resp.text)
            else:
                if os.path.splitext(data_input)[1] == '.json':
                    with open(data_input,'r') as fobj:
                        data = json.loads(fobj.read().replace("'",'"'))
                if os.path.splitext(data_input)[1] == '.hjson':
                    with open(data_input,'r') as fobj:
                        data = hjson.loads(fobj.read().replace("'",'"'))
        elif type(data_input) == dict:
            data = data_input
            
        self.data = data

        if not 'lines' in self.data:
            self.data['lines'] = []
        if not 'shunts' in self.data:
            self.data['shunts'] = []
        if not 'transformers' in self.data:
            self.data['transformers'] = []
        if not 'loads' in self.data:
            self.data['loads'] = []

        self.system = data['system']
        self.buses = data['buses']
        self.lines = data['lines']
        self.shunts = data['shunts']
        self.transformers = data['transformers']
        self.loads = data['loads']

        self.x_grid = []
        self.f_grid = []
    
        self.params_grid = {'S_base':self.system['S_base']}
        self.S_base = sym.Symbol("S_base", real=True) 
        self.N_bus = len(self.buses)
        self.N_branch = 3*len(self.lines) + len(self.shunts) + 2*len(self.transformers)
 
        self.dae = {'f':[],'g':[],'x':[],'y_ini':[],'y_run':[],
                    'u_ini_dict':{},'u_run_dict':{},'params_dict':{},
                    'h_dict':{},'xy_0_dict':{}}

        self.uz_jacs = True     
        self.verbose = False   
        self.testing = testing
                
    def contruct_grid(self):
        
        N_branch = self.N_branch
        N_bus = self.N_bus
        sys = self.system
        
        S_base = sym.Symbol('S_base', real=True)
        
        xy_0_dict_grid = {}
        u_grid = {}
        h_grid = {}
        
        A = sym.zeros(N_branch,N_bus)
        G_primitive = sym.zeros(N_branch,N_branch)
        B_primitive = sym.zeros(N_branch,N_branch)
        buses_list = [bus['name'] for bus in self.buses]
        it = 0
        for line in self.lines:
    
            bus_j = line['bus_j']
            bus_k = line['bus_k']
    
            idx_j = buses_list.index(bus_j)
            idx_k = buses_list.index(bus_k)    
    
            A[it,idx_j] = 1
            A[it,idx_k] =-1   
            A[it+1,idx_j] = 1
            A[it+2,idx_k] = 1   
            
            line_name = f"{bus_j}_{bus_k}"
            g_jk = sym.Symbol(f"g_{line_name}", real=True) 
            b_jk = sym.Symbol(f"b_{line_name}", real=True) 
            bs_jk = sym.Symbol(f"bs_{line_name}", real=True) 
            G_primitive[it,it] = g_jk
            B_primitive[it,it] = b_jk
            B_primitive[it+1,it+1] = bs_jk/2
            B_primitive[it+2,it+2] = bs_jk/2

            
            if 'X_pu' in line:
                if 'S_mva' in line: S_line = 1e6*line['S_mva']
                R = line['R_pu']*sys['S_base']/S_line  # in pu of the system base
                X = line['X_pu']*sys['S_base']/S_line  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_{line_name}":G})
                self.params_grid.update({f'b_{line_name}':B})
    
            if 'X' in line:
                bus_idx = buses_list.index(line['bus_j'])
                U_base = self.buses[bus_idx]['U_kV']*1000
                Z_base = U_base**2/sys['S_base']
                R = line['R']/Z_base  # in pu of the system base
                X = line['X']/Z_base  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_{line_name}":G})
                self.params_grid.update({f'b_{line_name}':B})
    
            if 'X_km' in line:
                bus_idx = buses_list.index(line['bus_j'])
                U_base = self.buses[bus_idx]['U_kV']*1000
                Z_base = U_base**2/sys['S_base']
                R = line['R_km']*line['km']/Z_base  # in pu of the system base
                X = line['X_km']*line['km']/Z_base  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_{line_name}":G})
                self.params_grid.update({f'b_{line_name}':B})        
    
            self.params_grid.update({f'bs_{line_name}':0.0})
            if 'Bs_pu' in line:
                if 'S_mva' in line: S_line = 1e6*line['S_mva']
                Bs = line['Bs_pu']*S_line/sys['S_base']  # in pu of the system base
                bs = Bs
                self.params_grid[f'bs_{line_name}'] = bs
     
            if 'Bs_km' in line:
                bus_idx = buses_list.index(line['bus_j'])
                U_base = self.buses[bus_idx]['U_kV']*1000
                Z_base = U_base**2/sys['S_base']
                Y_base = 1.0/Z_base
                Bs = line['Bs_km']*line['km']/Y_base # in pu of the system base
                bs = Bs 
                self.params_grid[f'bs_{line_name}'] = bs
                
            it += 3

        for trafo in self.transformers:
    
            bus_j = trafo['bus_j']
            bus_k = trafo['bus_k']
    
            idx_j = buses_list.index(bus_j)
            idx_k = buses_list.index(bus_k)    
    
            A[it,idx_j] = 1
            A[it+1,idx_k] = 1  
            
            trafo_name = f"{bus_j}_{bus_k}"
            g_jk = sym.Symbol(f"g_cc_{trafo_name}", real=True) 
            b_jk = sym.Symbol(f"b_cc_{trafo_name}", real=True) 
            tap = sym.Symbol(f"tap_{trafo_name}", real=True) 
            ang = sym.Symbol(f"ang_{trafo_name}", real=True) 
            a_s = tap*sym.cos(ang)
            b_s = tap*sym.sin(ang)

            Y_cc = g_jk + sym.I*b_jk

            
            Y_primitive =sym.Matrix([[ Y_cc/(a_s**2+b_s**2),-Y_cc/(a_s-sym.I*b_s)],
                                     [-Y_cc/(a_s+sym.I*b_s),                 Y_cc]])


            G_primitive[it:it+2,it:it+2] = sym.re(Y_primitive)
            B_primitive[it:it+2,it:it+2] = sym.im(Y_primitive)


            if 'X_pu' in trafo:
                if 'S_mva' in trafo: S_trafo = 1e6*trafo['S_mva']
                R = trafo['R_pu']*sys['S_base']/S_trafo  # in pu of the system base
                X = trafo['X_pu']*sys['S_base']/S_trafo  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_cc_{trafo_name}":G})
                self.params_grid.update({f'b_cc_{trafo_name}':B})
                tap_m = 1.0
                if 'tap_m' in trafo:
                    tap_m = trafo['tap_m']
                self.params_grid.update({f'tap_{trafo_name}':tap_m})
                self.params_grid.update({f'ang_{trafo_name}':0.0})

    
            if 'X' in trafo:
                bus_idx = buses_list.index(trafo['bus_j'])
                U_base = self.buses[bus_idx]['U_kV']*1000
                Z_base = U_base**2/sys['S_base']
                R = trafo['R']/Z_base  # in pu of the system base
                X = trafo['X']/Z_base  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_cc_{trafo_name}":G})
                self.params_grid.update({f'b_cc_{trafo_name}':B})
                self.params_grid.update({f'tap_{trafo_name}':1.0})
                self.params_grid.update({f'ang_{trafo_name}':0.0})
     
            it += 2


        for shunt in self.shunts:
    
            bus_j = shunt['bus']   
            idx_j = buses_list.index(bus_j)
    
            A[it,idx_j] = 1
             
            shunt_name = f"{bus_j}"
            g_j = sym.Symbol(f"g_shunt_{shunt_name}", real=True) 
            b_j = sym.Symbol(f"b_shunt_{shunt_name}", real=True) 
            G_primitive[it,it] = g_j
            B_primitive[it,it] = b_j

            
            if 'X_pu' in shunt:
                if 'S_mva' in shunt: S_line = 1e6*shunt['S_mva']
                R = shunt['R_pu']*sys['S_base']/S_line  # in pu of the system base
                X = shunt['X_pu']*sys['S_base']/S_line  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_shunt_{shunt_name}":G})
                self.params_grid.update({f'b_shunt_{shunt_name}':B})
    
            if 'X' in shunt:
                U_base = self.buses[idx_j]['U_kV']*1000
                Z_base = U_base**2/sys['S_base']
                R = shunt['R']/Z_base  # in pu of the system base
                X = shunt['X']/Z_base  # in pu of the system base
                G =  R/(R**2+X**2)
                B = -X/(R**2+X**2)
                self.params_grid.update({f"g_shunt_{shunt_name}":G})
                self.params_grid.update({f'b_shunt_{shunt_name}':B})
                
            it += 1    
    
        G = A.T * G_primitive * A
        B = A.T * B_primitive * A    
        
        sin = sym.sin
        cos = sym.cos
        y_grid = []
        g = sym.zeros(2*N_bus,1)

        for j in range(N_bus):
            bus_j_name = buses_list[j]
            P_j = sym.Symbol(f"P_{bus_j_name}", real=True)
            Q_j = sym.Symbol(f"Q_{bus_j_name}", real=True)
            g[2*j]   = -P_j/S_base
            g[2*j+1] = -Q_j/S_base
            for k in range(N_bus): 
    
                bus_k_name = buses_list[k]
                V_j = sym.Symbol(f"V_{bus_j_name}", real=True) 
                V_k = sym.Symbol(f"V_{bus_k_name}", real=True) 
                theta_j = sym.Symbol(f"theta_{bus_j_name}", real=True) 
                theta_k = sym.Symbol(f"theta_{bus_k_name}", real=True) 
                g[2*j]   += V_j*V_k*(G[j,k]*cos(theta_j - theta_k) + B[j,k]*sin(theta_j - theta_k)) 
                g[2*j+1] += V_j*V_k*(G[j,k]*sin(theta_j - theta_k) - B[j,k]*cos(theta_j - theta_k))        
                h_grid.update({f"V_{bus_j_name}":V_j})
            bus = self.buses[j]
            bus_name = bus['name']
            if 'type' in bus:
                if bus['type'] == 'slack':
                    y_grid += [P_j]
                    y_grid += [Q_j]
                    u_grid.update({f"V_{bus_name}":1.0})
                    u_grid.update({f"theta_{bus_name}":0.0})  
            else:
                y_grid += [V_j]
                y_grid += [theta_j]        
                u_grid.update({f"P_{bus_name}":bus['P_W']})
                u_grid.update({f"Q_{bus_name}":bus['Q_var']})    
                xy_0_dict_grid.update({str(V_j):1.0,str(theta_j):0.0})
                
            self.params_grid.update({f'U_{bus_name}_n':bus['U_kV']*1000})
        g_grid = list(g)     
    
        if False:
            v_sym_list = []
            for bus in buses_list:
                V_m = sym.Symbol(f'V_{bus}',real=True)
                V_a = sym.Symbol(f'theta_{bus}',real=True)
                v_sym_list += [V_m*sym.exp(sym.I*V_a)]
    
            sym.Matrix(v_sym_list)
    
            I_lines = (G_primitive+1j*B_primitive) * A * sym.Matrix(v_sym_list)
    
            it = 0
            for line in self.lines:
                I_jk_r = sym.Symbol(f"I_{line['bus_j']}_{line['bus_k']}_r", real=True)
                I_jk_i = sym.Symbol(f"I_{line['bus_j']}_{line['bus_k']}_i", real=True)
                g_grid += [-I_jk_r + sym.re(I_lines[it])]
                g_grid += [-I_jk_i + sym.im(I_lines[it])]
                y_grid += [I_jk_r]
                y_grid += [I_jk_i]
                it += 1
        
        for line in self.lines:
    
            bus_j = line['bus_j']
            bus_k = line['bus_k']

            line_name = f"{bus_j}_{bus_k}"
    
            idx_j = buses_list.index(bus_j)
            idx_k = buses_list.index(bus_k)  

            V_j = sym.Symbol(f"V_{bus_j}", real=True) 
            V_k = sym.Symbol(f"V_{bus_k}", real=True) 
            theta_j = sym.Symbol(f"theta_{bus_j}", real=True) 
            theta_k = sym.Symbol(f"theta_{bus_k}", real=True)

            b_ij_p = 0.0
            if f'bs_{line_name}' in self.params_grid:
                b_ij_p = self.params_grid[f'bs_{line_name}']

            G_jk = G[idx_j,idx_k] 
            B_jk = B[idx_j,idx_k] 
            theta_jk = theta_j - theta_k
            P_line_to   = V_j*V_k*(G_jk*sym.cos(theta_jk) + B_jk*sym.sin(theta_jk)) - V_j**2*(G_jk) 
            Q_line_to   = V_j*V_k*(G_jk*sym.sin(theta_jk) - B_jk*sym.cos(theta_jk)) + V_j**2*(B_jk) 
            P_line_from = V_j*V_k*(G_jk*sym.cos(-theta_jk) + B_jk*sym.sin(-theta_jk)) - V_k**2*(G_jk) 
            Q_line_from = V_j*V_k*(G_jk*sym.sin(-theta_jk) - B_jk*sym.cos(-theta_jk)) + V_k**2*(B_jk) 

            if 'monitor' in line:
                if line['monitor']:
                    h_grid.update({f"p_line_{bus_j}_{bus_k}":P_line_to})
                    h_grid.update({f"q_line_{bus_j}_{bus_k}":Q_line_to}) 
                    h_grid.update({f"p_line_{bus_k}_{bus_j}":P_line_from})
                    h_grid.update({f"q_line_{bus_k}_{bus_j}":Q_line_from}) 

        self.dae['f'] += []
        self.dae['g'] += g_grid
        self.dae['x'] += []
        self.dae['y_ini'] += y_grid
        self.dae['y_run'] += y_grid
        self.dae['u_ini_dict'].update(u_grid)
        self.dae['u_run_dict'].update(u_grid)
        self.dae['h_dict'].update(h_grid)
        self.dae['params_dict'].update(self.params_grid)
        self.dae['xy_0_dict'].update(xy_0_dict_grid)
        
        self.A_incidence = A
        self.G_primitive = G_primitive
        self.B_primitive = B_primitive


        self.N_syn = 0
        self.N_gformers = 0
        
        self.generators_list = []
        self.generators_id_list = []
        
        # COI 
        omega_coi = sym.Symbol("omega_coi", real=True)  
        
        self.H_total = 0
        self.omega_coi_numerator = 0.0
        self.omega_coi_denominator = 0.0

        self.dae['xy_0_dict'].update({str(omega_coi):1.0})  

        self.buses = self.data['buses']
        self.buses_list = [bus['name'] for bus in self.buses]  
        

    def construct(self, name):
        
        self.contruct_grid()   
        
#        omega_coi = sym.Symbol("omega_coi", real=True)  
#        
#        self.dae['g'] += [ -omega_coi + self.omega_coi_numerator/self.omega_coi_denominator]
#        self.dae['y_ini'] += [ omega_coi]
#        self.dae['y_run'] += [ omega_coi]        
        


        #grid = bmapu_builder.bmapu(data)
        if 'syns' in self.data:
            add_syns(self)
        if 'vscs' in self.data:
            add_vscs(self)
        if 'vsgs' in self.data:
            add_vsgs(self)
        if 'sources' in  self.data:
            add_sources(self)
        if 'wecs' in  self.data:
            add_wecs(self)
        if 'pvs' in  self.data:
            add_pvs(self)
        if 'loads' in  self.data:
            add_loads(self)
            
        add_miscellaneous(self)

        #add_vsgs(grid)
        omega_coi = sym.Symbol("omega_coi", real=True)  

        self.dae['g'] += [ -omega_coi + self.omega_coi_numerator/self.omega_coi_denominator]
        self.dae['y_ini'] += [ omega_coi]
        self.dae['y_run'] += [ omega_coi]

        # secondary frequency control
        xi_freq = sym.Symbol("xi_freq", real=True) 
        p_agc = sym.Symbol("p_agc", real=True)  
        K_p_agc = sym.Symbol("K_p_agc", real=True) 
        K_i_agc = sym.Symbol("K_i_agc", real=True) 
        K_xif  = sym.Symbol("K_xif", real=True)

        epsilon_freq = 1-omega_coi
        g_agc = [ -p_agc + K_p_agc*epsilon_freq + K_i_agc*xi_freq ]
        y_agc = [  p_agc]
        x_agc = [ xi_freq]
        f_agc = [epsilon_freq - K_xif*xi_freq]

        self.dae['g'] += g_agc
        self.dae['y_ini'] += y_agc
        self.dae['y_run'] += y_agc
        self.dae['f'] += f_agc
        self.dae['x'] += x_agc
        self.dae['params_dict'].update({'K_p_agc':self.system['K_p_agc'],'K_i_agc':self.system['K_i_agc']})

        if 'K_xif' in self.system:
            self.dae['params_dict'].update({'K_xif':self.system['K_xif']})
        else:
            self.dae['params_dict'].update({'K_xif':0.0})

         
        with open('xy_0.json','w') as fobj:
            fobj.write(json.dumps(self.dae['xy_0_dict'],indent=4))

        with open(f'{name}_xy_0.json','w') as fobj:
            fobj.write(json.dumps(self.dae['xy_0_dict'],indent=4))

        self.sys_dict = {'name':name,'uz_jacs':self.uz_jacs,
                'params_dict':self.dae['params_dict'],
                'f_list':self.dae['f'],
                'g_list':self.dae['g'] ,
                'x_list':self.dae['x'],
                'y_ini_list':self.dae['y_ini'],
                'y_run_list':self.dae['y_run'],
                'u_run_dict':self.dae['u_run_dict'],
                'u_ini_dict':self.dae['u_ini_dict'],
                'h_dict':self.dae['h_dict']}
        if self.testing:
            self.sys_dict.update({'testing':True})


    def compile(self):

        bldr = db.builder(self.sys_dict,verbose=self.verbose);
        bldr.build()    

    def compile_mkl(self, name):

        b = builder(self.sys_dict,verbose=self.verbose)
        b.sparse = True
        b.mkl = True
        b.uz_jacs = False
        b.dict2system()
        b.functions()
        b.jacobians()
        b.cwrite()
        b.template()
        b.compile_mkl()  

    def build(self, name =''):
        if name == '':
            print('Error: name is not provided.')
        self.construct(name)    
        self.compile()  

    def checker(self):
        
        if not 'syns' in self.data: self.data.update({'syns':[]})
        if not 'vscs' in self.data: self.data.update({'vscs':[]})
        if not 'vsgs' in self.data: self.data.update({'vsgs':[]})
        if not 'genapes' in self.data: self.data.update({'genapes':[]})
        if not 'sources' in self.data: self.data.update({'sources':[]})
                                                         
                                                         
        K_deltas_n = 0
        for item in self.data['syns']:
            if item['K_delta'] > 0.0:
                K_deltas_n += 1
        for item in self.data['vscs']:
            if item['K_delta'] > 0.0:
                K_deltas_n += 1
        for item in self.data['vsgs']:
            if item['K_delta'] > 0.0:
                K_deltas_n += 1
        for item in self.data['genapes']:
            if item['K_delta'] > 0.0:
                K_deltas_n += 1      
        for item in self.data['sources']:
            if item['type'] > 'vsource':
                K_deltas_n += 1   

        if  K_deltas_n == 0:
            print('One generator must have K_delta > 0.0')
        if  K_deltas_n > 1:
            print('Only one generator must have K_delta > 0.0')                                          
                                                      
        if len(self.data['genapes']) > 0:
            if self.data['system']['K_p_agc'] != 0.0:
                print('With a genape in the system K_p_agc must be set to 0.0')
            if self.data['system']['K_i_agc'] != 0.0:
                print('With a genape in the system K_i_agc must be set to 0.0')  
            if not self.data['system']['K_xif'] > 0.0:
                print('With a genape in the system K_xif must be set larger than 0.0')     


        for item in self.data['syns']:
            if 'gov' in item:
                if 'K_imw' in item['gov']:
                    if item['gov']['p_c'] > 0.0 and item['gov']['K_imw'] == 0.0:
                        print(f"Synchornous machine at bus {item['bus']} has p_c > 0, therefore K_imw should be larger than 0")                
            
        
        for item in self.data['syns']:
            if 'avr' in item:
                if 'sexs' in item['avr']:
                    if item['avr']['K_ai'] <= 0.0:
                        print(f"AVR of a synchornous machine at bus {item['bus']} must have constant K_ai larger than 0")                
            
               
            

if __name__ == "__main__":

    data = {
        "sys":{"name":"k12p6","S_base":100e6, "K_p_agc":0.01,"K_i_agc":0.01},       
        "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
                 {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0}],
        "lines":[{"bus_j":"1", "bus_k":"2", "X_pu":0.15,"R_pu":0.0, "S_mva":900.0}]
        }

    grid = bmapu(data)

    from syns.syns import add_syns
    add_syns(grid,'hola')
    
    # data = {
    #     "sys":{"name":"k12p6","S_base":100e6, "K_p_agc":0.01,"K_i_agc":0.01},       
    #     "buses":[{"name":"1", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
    #              {"name":"2", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
    #              {"name":"3", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
    #              {"name":"4", "P_W":0.0,"Q_var":0.0,"U_kV":20.0},
    #              {"name":"5", "P_W":0.0,"Q_var":0.0,"U_kV":230.0},
    #              {"name":"6", "P_W":0.0,"Q_var":0.0,"U_kV":230.0},
    #              {"name":"7", "P_W":-967e6,"Q_var":100e6,"U_kV":230.0},
    #              {"name":"8", "P_W":0.0,"Q_var":0.0,"U_kV":230.0},
    #              {"name":"9", "P_W":-1767e6,"Q_var":250e6,"U_kV":230.0},
    #              {"name":"10","P_W":0.0,"Q_var":0.0,"U_kV":230.0},
    #              {"name":"11","P_W":0.0,"Q_var":0.0,"U_kV":230.0}      
    #             ],
    #     "lines":[{"bus_j":"1", "bus_k":"5", "X_pu":0.15,"R_pu":0.0, "S_mva":900.0},
    #              {"bus_j":"2", "bus_k":"6", "X_pu":0.15,"R_pu":0.0, "S_mva":900.0},
    #              {"bus_j":"3", "bus_k":"11","X_pu":0.15,"R_pu":0.0, "S_mva":900.0},
    #              {"bus_j":"4", "bus_k":"10","X_pu":0.15,"R_pu":0.0, "S_mva":900.0},
    #              {"bus_j":"5", "bus_k":"6", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":25},
    #              {"bus_j":"6", "bus_k":"7", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":10},
    #              {"bus_j":"7", "bus_k":"8", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":110},
    #              {"bus_j":"7", "bus_k":"8", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":110},
    #              {"bus_j":"8", "bus_k":"9", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":110},
    #              {"bus_j":"8", "bus_k":"9", "X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":110},
    #              {"bus_j":"9", "bus_k":"10","X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":10},
    #              {"bus_j":"10","bus_k":"11","X_km":0.529,"R_km":0.0529,"Bs_km":2.1e-6,"km":25}]
    #     }
    
    # bpu_obj = bpu(data_input=data)
    


if __name__ == "__main__":

    
    params_dict = {'X_d':1.81,'X1d':0.3,'T1d0':8.0,
                   'X_q':1.76,'X1q':0.65,'T1q0':1.0,
                   'R_a':0.003,'X_l': 0.05, 
                   'H':3.5,'D':1.0,
                   'Omega_b':2*np.pi*50,'omega_s':1.0,
                   'v_0':0.9008,'theta_0':0.0}
    
    
    u_ini_dict = {'P_t':0.8, 'Q_t':0.2}  # for the initialization problem
    u_run_dict = {'p_m':0.8,'e1q':1.0}  # for the running problem (here initialization and running problem are the same)
    
    
    x_list = ['delta','omega']    # [inductor current, PI integrator]
    y_ini_list = ['i_d','i_q','v_1','theta_1','p_m','e1q'] # for the initialization problem
    y_run_list = ['i_d','i_q','v_1','theta_1','P_t','Q_t'] # for the running problem (here initialization and running problem are the same)
    
    sys_vars = {'params':params_dict,
                'u_list':u_run_dict,
                'x_list':x_list,
                'y_list':y_run_list}
    
    exec(sym_gen_str())  # exec to generate the required symbolic varables and constants
    






import numpy as np
import sympy as sym

def acdc_3ph_4w_l(grid,vsc_data):
    '''
    VSC with 3 phase and 4 wire working in open loop as a grid former.
    
    '''

    params_dict  = grid.dae['params_dict']
    f_list = grid.dae['f']
    x_list = grid.dae['x']
    g_list = grid.dae['g'] 
    y_ini_list = grid.dae['y_ini'] 
    u_ini_dict = grid.dae['u_ini_dict']
    y_run_list = grid.dae['y_run'] 
    u_run_dict = grid.dae['u_run_dict']
    h_dict = grid.dae['h_dict']


    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    omega_coi = sym.Symbol('omega_coi',real=True)
    xi_freq = sym.Symbol('xi_freq',real=True)

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
        
    name_ac = vsc_data['ac_bus']
    name_dc = vsc_data['dc_bus']

    # inputs
    v_dc = sym.Symbol(f'v_dc_{name_ac}', real=True)
    m_a,m_b,m_c,m_n = sym.symbols(f'm_a_{name_ac},m_b_{name_ac},m_c_{name_ac},m_n_{name_ac}', real=True)
    phi = sym.Symbol(f'phi_{name_ac}', real=True)
    phi_a = sym.Symbol(f'phi_a_{name_ac}', real=True)
    phi_b = sym.Symbol(f'phi_b_{name_ac}', real=True)
    phi_c = sym.Symbol(f'phi_c_{name_ac}', real=True)
    phi_n = sym.Symbol(f'phi_n_{name_ac}', real=True)

    # parameters
    R_s,R_sn,R_ng = sym.symbols(f'R_{name_ac}_s,R_{name_ac}_sn,R_{name_ac}_ng', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_{name_ac}_s,X_{name_ac}_sn,X_{name_ac}_ng', real=True)
    
    # dynamical states
    De_ao_m,De_bo_m,De_co_m,De_no_m  = sym.symbols(f'De_ao_m_{name_ac},De_bo_m_{name_ac},De_co_m_{name_ac},De_no_m_{name_ac}', real=True)


    omega = sym.Symbol(f'omega_{name_ac}', real=True)
    
    # algebraic states

    ## ac side
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name_ac}_0_r,V_{name_ac}_1_r,V_{name_ac}_2_r,V_{name_ac}_3_r,v_{name_ac}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name_ac}_0_i,V_{name_ac}_1_i,V_{name_ac}_2_i,V_{name_ac}_3_i,v_{name_ac}_o_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name_ac}_a_r,i_vsc_{name_ac}_b_r,i_vsc_{name_ac}_c_r,i_vsc_{name_ac}_n_r,i_vsc_{name_ac}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name_ac}_a_i,i_vsc_{name_ac}_b_i,i_vsc_{name_ac}_c_i,i_vsc_{name_ac}_n_i,i_vsc_{name_ac}_ng_i', real=True)
    p_dc,i_dc = sym.symbols(f'p_dc_{name_ac},i_dc_{name_ac}', real=True)

    Z_sa = R_s + 1j*X_s
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i

    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_og = v_og_r + 1j*v_og_i

    s_sa = v_sa * sym.conjugate(i_sa)
    s_sb = v_sb * sym.conjugate(i_sb)
    s_sc = v_sc * sym.conjugate(i_sc)
    s_sn = v_sn * sym.conjugate(i_sc)



    sqrt6 = np.sqrt(6)
    e_ao_r = (m_a*v_dc/sqrt6)*sym.cos(phi_a + phi) 
    e_ao_i = (m_a*v_dc/sqrt6)*sym.sin(phi_a + phi) 
    e_bo_r = (m_b*v_dc/sqrt6)*sym.cos(phi_b + phi-2/3*np.pi) 
    e_bo_i = (m_b*v_dc/sqrt6)*sym.sin(phi_b + phi-2/3*np.pi) 
    e_co_r = (m_c*v_dc/sqrt6)*sym.cos(phi_c + phi-4/3*np.pi) 
    e_co_i = (m_c*v_dc/sqrt6)*sym.sin(phi_c + phi-4/3*np.pi) 
    e_no_r = (m_n*v_dc/sqrt6)*sym.cos(phi_n) 
    e_no_i = (m_n*v_dc/sqrt6)*sym.sin(phi_n) 

    e_ao_cplx = e_ao_r + 1j*e_ao_i
    e_bo_cplx = e_bo_r + 1j*e_bo_i
    e_co_cplx = e_co_r + 1j*e_co_i
    e_no_cplx = e_no_r + 1j*e_no_i

    s_ta = v_sa * sym.conjugate(i_sa)
    s_tb = v_sb * sym.conjugate(i_sb)
    s_tc = v_sc * sym.conjugate(i_sc)
    s_tn = v_sn * sym.conjugate(i_sc)

    p_ac_a = sym.re(s_ta)
    p_ac_b = sym.re(s_tb)
    p_ac_c = sym.re(s_tc)
    p_ac_n = sym.re(s_tn)

    v_san = v_sa - v_sn
    v_sbn = v_sb - v_sn
    v_scn = v_sc - v_sn

    eq_i_sa_cplx = v_og + e_ao_cplx - i_sa*Z_sa - v_sa   # v_sa = v_sag
    eq_i_sb_cplx = v_og + e_bo_cplx - i_sb*Z_sb - v_sb
    eq_i_sc_cplx = v_og + e_co_cplx - i_sc*Z_sc - v_sc
    eq_i_sn_cplx = v_og + e_no_cplx - i_sn*Z_sn - v_sn
    eq_v_og_cplx = i_sa + i_sb + i_sc + i_sn + v_og/Z_ng

    ### DC side
    v_sp_r,v_sm_r = sym.symbols(f'V_{name_dc}_0_r,V_{name_dc}_1_r', real=True)
    v_sp_i,v_sm_i = sym.symbols(f'V_{name_dc}_0_i,V_{name_dc}_1_i', real=True)

    A_loss,B_loss,C_loss = sym.symbols(f'A_loss_{name_ac},B_loss_{name_ac},C_loss_{name_ac}',real=True)
    i_rms_a = sym.sqrt(i_sa_r**2+i_sa_i**2+1e-6) 
    i_rms_b = sym.sqrt(i_sb_r**2+i_sb_i**2+1e-6) 
    i_rms_c = sym.sqrt(i_sc_r**2+i_sc_i**2+1e-6) 
    i_rms_n = sym.sqrt(i_sn_r**2+i_sn_i**2+1e-6) 

    p_loss_a = A_loss + B_loss*i_rms_a + C_loss*i_rms_a*i_rms_a
    p_loss_b = A_loss + B_loss*i_rms_b + C_loss*i_rms_b*i_rms_b
    p_loss_c = A_loss + B_loss*i_rms_c + C_loss*i_rms_c*i_rms_c
    p_loss_n = A_loss + B_loss*i_rms_n + C_loss*i_rms_n*i_rms_n

    p_vsc_loss = p_loss_a + p_loss_b + p_loss_c + p_loss_n
    p_ac = p_ac_a + p_ac_b + p_ac_c + p_ac_n

    eq_p_dc = -p_dc + p_ac + p_vsc_loss
    eq_i_dc = p_dc/v_dc - i_dc
 


    g_list += [sym.re(eq_i_sa_cplx)] 
    g_list += [sym.re(eq_i_sb_cplx)] 
    g_list += [sym.re(eq_i_sc_cplx)] 
    g_list += [sym.re(eq_i_sn_cplx)] 
    g_list += [sym.re(eq_v_og_cplx)] 
    g_list += [sym.im(eq_i_sa_cplx)] 
    g_list += [sym.im(eq_i_sb_cplx)] 
    g_list += [sym.im(eq_i_sc_cplx)] 
    g_list += [sym.im(eq_i_sn_cplx)] 
    g_list += [sym.im(eq_v_og_cplx)] 
    g_list += [eq_p_dc,eq_i_dc]

    y_ini_list += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r,p_dc,i_dc]
    y_ini_list += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]
    y_run_list += [i_sa_r,i_sb_r,i_sc_r,i_sn_r,v_og_r,p_dc,i_dc]
    y_run_list += [i_sa_i,i_sb_i,i_sc_i,i_sn_i,v_og_i]


    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name_ac}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name_ac}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name_ac,ph)
        grid.dae['g'] [idx_r] += -i_s_r
        grid.dae['g'] [idx_i] += -i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        h_dict.update({f'i_vsc_{name_ac}_{ph}_m':i_s_m})

    idx_r = grid.node2idx(name_dc,'0')
    grid.dae['g'] [idx_r] += i_dc
    idx_r = grid.node2idx(name_dc,'1')
    grid.dae['g'] [idx_r] += -i_dc
    h_dict.update({f'i_vsc_{name_dc}':i_dc})

    #    V_1 = 400/np.sqrt(3)*np.exp(1j*np.deg2rad(0))
    # A_1toabc = np.array([1, alpha**2, alpha])
    #V_abc = V_1 * A_1toabc 
    #e_an_r,e_bn_r,e_cn_r = V_abc.real
    #e_an_i,e_bn_i,e_cn_i = V_abc.imag

    u_ini_dict.update({f'v_dc_{name_ac}':800.0})
    u_run_dict.update({f'v_dc_{name_ac}':800.0})
    m = 0.7071
    u_ini_dict.update({f'm_a_{name_ac}':m,f'm_b_{name_ac}':m,f'm_c_{name_ac}':m,f'm_n_{name_ac}':0.0})
    u_run_dict.update({f'm_a_{name_ac}':m,f'm_b_{name_ac}':m,f'm_c_{name_ac}':m,f'm_n_{name_ac}':0.0})

    u_ini_dict.update({f'phi_{name_ac}':0.0})
    u_ini_dict.update({f'phi_a_{name_ac}':0.0})
    u_ini_dict.update({f'phi_b_{name_ac}':0.0})
    u_ini_dict.update({f'phi_c_{name_ac}':0.0})
    u_ini_dict.update({f'phi_n_{name_ac}':0.0})

    u_run_dict.update({f'phi_{name_ac}':0.0})
    u_run_dict.update({f'phi_a_{name_ac}':0.0})
    u_run_dict.update({f'phi_b_{name_ac}':0.0})
    u_run_dict.update({f'phi_c_{name_ac}':0.0})
    u_run_dict.update({f'phi_n_{name_ac}':0.0})


    params_dict.update({f'X_{name_ac}_s':vsc_data['X'],f'R_{name_ac}_s':vsc_data['R']})
    params_dict.update({f'X_{name_ac}_sn':vsc_data['X_n'],f'R_{name_ac}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name_ac}_ng':vsc_data['X_ng'],f'R_{name_ac}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name_ac}':vsc_data['S_n']})
   
    
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    
    v_szpn = A_a0*v_sabc
    i_szpn = A_a0*i_sabc
    
    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])

    h_dict.update({f'p_{name_ac}_pos':sym.re(s_pos),f'p_{name_ac}_neg':sym.re(s_neg),f'p_{name_ac}_zer':sym.re(s_zer)})
    h_dict.update({str(m_a):m_a,str(m_b):m_b,str(m_c):m_c,str(m_n):m_n})
    h_dict.update({str(phi):phi})

    S_n_num = vsc_data['S_n']
    U_n_num = vsc_data['U_n']
    I_n = S_n_num/(np.sqrt(3)*U_n_num)
    P_0 = 0.01*S_n_num
    A_loss_num = P_0/3
    P_cc = 0.01*S_n_num
    C_loss_num = P_cc/(I_n**2)/3

    params_dict.update({str(A_loss):A_loss_num})
    params_dict.update({str(B_loss):0.0})
    params_dict.update({str(C_loss):C_loss_num})





# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def aero(grid,name,bus_name,data_dict):
    '''
    ## Aerodinamic


    parameters
    ----------

    S_n: nominal power in VA
    F_n: nominal frequency in Hz
    X_v: coupling reactance in pu (base machine S_n)
    R_v: coupling resistance in pu (base machine S_n)
    K_delta: if K_delta>0.0 current generator is converted to reference machine 
    K_alpha: alpha gain to obtain Domega integral 

    inputs
    ------

    alpha: RoCoF in pu if K_alpha = 1.0
    omega_ref: frequency in pu
    v_ref: internal voltage reference

    example
    -------

    "vscs": [{"type":"vsc_l","S_n":1e6,"F_n":50.0,"X_s":0.1,"R_s":0.01}]

    v_t = m*v_dc/sqrt(6)

    v_t_pu*v_b_ac = m*v_dc_pu*v_b_dc/sqrt(6)

    S_b = S_n
    U_b = U_n
    Omega_b = 2*np.pi*F_b
    Omega_rb = Omega_rn
    Tau_b = S_b/Omega_rb
    V_dc_b = sqrt(2)*U_b
    I_b = (np.sqrt(3)*U_b)
    I_dq_b = I_b*np.sqrt(2)
    V_dq_b = U_b*np.sqrt(2/3)

    V_dq_b and I_dq_b defined as below verify that: 
    3/2*I_dq_b*V_dq_b = 3/2*I_b*np.sqrt(2)*U_b*np.sqrt(2/3) = np.sqrt(3)*U_b*I_b

    Z_b = U_b**2/S_b
    Z_b = (V_dq_b/np.sqrt(2/3))**2/S_b = 3/2*V_dq_b**2/S_b
    L_m_b = Z_b/Omega_rb

    tau_r = 3/2*Phi*N_pp*i_mq

    tau_r_pu = (3/2*Phi*N_pp*i_mq_pu)*I_dq_b/Tau_b  = K_tau * i_mq_pu
    with K_tau = 3/2*Phi*N_pp*I_dq_b/Tau_b

    0 =               - R_s*i_md - omega_r*L_m*i_mq - v_md  
    0 = omega_r*Phi_m - R_s*i_mq + omega_r*L_m*i_md - v_mq 

    0 =               (- R_s*i_md_pu/V_dq_b - omega_r*L_m*i_mq_pu/V_dq_b - v_md_pu/I_dq_b)*I_dq_b*V_dq_b
    0 =               (- R_s_pu*i_md_pu/V_dq_b - omega_r*L_m_pu*i_mq_pu*Z_b/Omega_rb/(V_dq_b*Z_b) - v_md_pu/(I_dq_b*Z_b))*I_dq_b*V_dq_b*Z_b
    0 =               (- R_s_pu*i_md_pu/V_dq_b - omega_r_pu*L_m_pu*i_mq_pu/(V_dq_b) - v_md_pu/(I_dq_b*Z_b))*I_dq_b*V_dq_b*Z_b


    v_t_pu*v_b_ac = m*V_dc_b*sqrt(2)*U_b/sqrt(6)
    v_t_pu = m*V_dc_b*sqrt(2)/sqrt(6) = m*V_dc_b*sqrt(3)


    

    '''


    # inputs
    dnu_w_ramp_ref = sym.Symbol(f"dnu_w_ramp_ref_{name}", real=True)
    omega_t  = sym.Symbol(f"omega_t_{name}", real=True)
    beta     = sym.Symbol(f"beta_{name}", real=True)
    nu_w_ramp= sym.Symbol(f"nu_w_ramp_{name}", real=True) 
    grid.dae['u_ini_dict'].update({f'dnu_w_ramp_ref_{name}':0.0})
    grid.dae['u_run_dict'].update({f'dnu_w_ramp_ref_{name}':0.0})  
    grid.dae['u_ini_dict'].update({f'nu_w_0_{name}':10.0})
    grid.dae['u_run_dict'].update({f'nu_w_0_{name}':10.0})  

    # algebraic states
    nu_w_0 = sym.Symbol(f"nu_w_0_{name}", real=True)
    
    # dynamic states
    nu_w_ramp = sym.Symbol(f"nu_w_ramp_{name}", real=True)

    # parameters
    C_1,C_2,C_3 = sym.symbols(f"C_1_{name},C_2_{name},C_3_{name}", real=True)
    C_4,C_5,C_6 = sym.symbols(f"C_4_{name},C_5_{name},C_6_{name}", real=True)
    Nu_w_b,Lam_b = sym.symbols(f"Nu_w_b_{name},Lam_b_{name}", real=True)
    Omega_t_b = sym.symbols(f"Omega_t_b_{name}", real=True)
    C_p_b,K_pow = sym.symbols(f"C_p_b_{name},K_pow_{name}", real=True)
    C_1 = 0.5176
    C_2 = 116
    C_3 = 0.4
    C_4 = 5.0
    C_5 = 21 
    C_6 = 0.0068
    Nu_w_b = 12
    Lam_b = 8.1
    Omega_t_b = 1.2
    K_pow = 1.0
    grid.dae['params_dict'].update({f"C_1_{name}":C_1})
    grid.dae['params_dict'].update({f"C_2_{name}":C_2})
    grid.dae['params_dict'].update({f"C_3_{name}":C_3})
    grid.dae['params_dict'].update({f"C_4_{name}":C_4})
    grid.dae['params_dict'].update({f"C_5_{name}":C_5})
    grid.dae['params_dict'].update({f"C_6_{name}":C_6})
    grid.dae['params_dict'].update({f"Nu_w_b_{name}":Nu_w_b})
    grid.dae['params_dict'].update({f"Lam_b_{name}":Lam_b})
    grid.dae['params_dict'].update({f"Omega_t_b_{name}":Omega_t_b})
    grid.dae['params_dict'].update({f"K_pow_{name}":K_pow})

    # auxiliar
    nu_w = nu_w_0 + nu_w_ramp
    lam = Lam_b*(omega_t/Omega_t_b)/(nu_w/Nu_w_b) # pu
    inv_lam_i =  1/(lam + 0.08*beta) - 0.035/(beta**3 + 1.0)   
    c_p = C_1*(C_2*inv_lam_i - C_3*beta - C_4)*sym.exp(-C_5*inv_lam_i) + C_6*lam 
    c_p_pu = c_p/C_p_b # (pu)
    p_w_nosat = K_pow*c_p_pu*(nu_w/Nu_w_b)**3 
    p_w = sym.Piecewise((0.0,p_w_nosat<0.0), (p_w_nosat,True)) 

    # dynamic
    dnu_w_ramp = dnu_w_ramp_ref - 0.0001*nu_w_ramp

    grid.dae['f'] += [dnu_w_ramp]
    grid.dae['x'] += [ nu_w_ramp]


    # outputs
    grid.dae['h_dict'].update({f"nu_w_{name}":nu_w})

    return p_w


def test_build():

    import sympy as sym
    from pydae.build_v2 import builder


    dae = {'f':[],'g':[],'x':[],'y_ini':[],'y_run':[],
                    'u_ini_dict':{},'u_run_dict':{},'params_dict':{},
                    'h_dict':{},'xy_0_dict':{},'name':'temp'}
    name = ''
    bus_name = ''
    data_dict = {}
    grid = type('grid_class', (object,), {'dae':dae})()

    
    p_w = aero(grid,name,bus_name,data_dict)

    sys_dict = {'name':'temp','uz_jacs':True,
                    'params_dict':grid.dae['params_dict'],
                    'f_list':grid.dae['f'],
                    'g_list':grid.dae['g'] ,
                    'x_list':grid.dae['x'],
                    'y_ini_list':grid.dae['y_ini'],
                    'y_run_list':grid.dae['y_run'],
                    'u_run_dict':grid.dae['u_run_dict'],
                    'u_ini_dict':grid.dae['u_ini_dict'],
                    'h_dict':grid.dae['h_dict']}
    b = builder(sys_dict,verbose=True)
    b.sparse = True
    b.mkl = True
    b.uz_jacs = True
    b.dict2system()
    b.functions()
    b.jacobians()
    b.cwrite()
    b.template()
    b.compile_mkl()  


if __name__ == '__main__':
    test_build()
    import temp
    model = temp.model()
    model.ini({})


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
2020

@author: jmmauricio
"""

import numpy as np
import json


def save(json_file,dictionary, sort_keys=False, indent=4):
    '''
    Convert dictionary to json and save it in json_file path.

    Parameters
    ----------
    json_file : path
         Path of the json data file..
    dictionary : dict
        Dictionary with the loaded data as a dictionary.

    Returns
    -------
    None.

    '''
    
    data_json = json.dumps(dictionary, sort_keys=False, indent=4)    
    with open(json_file, 'w') as fobj:
        fobj.write(data_json)

    

    
def load(json_file):
    '''
    Read json file and convert it to dictionary.

    Parameters
    ----------
    json_file : path
        Path of the json data file.

    Returns
    -------
    data_dictionary: dict
        Dictionary with the loaded data as a dictionary.

    '''
    
    with open(json_file, 'r') as fobj:
        json_data = fobj.read()
        
    json_data = json_data.replace("'",'"')
    data_dictionary = json.loads(json_data)        
    
    return data_dictionary



def get_v(syst,bus_name,phase_name='',v_type='rms_phph',dq_name='DQ'):
    
    if dq_name == 'DQ':
        D_ = 'D'
        Q_ = 'Q'    

        v_d_name = f'v_{bus_name}_{D_}'
        v_q_name = f'v_{bus_name}_{Q_}'

        v_d = syst.get_value(v_d_name)
        v_q = syst.get_value(v_q_name)

        v_m = 0.0
        if v_type == 'rms_phph':
            v_m = np.abs(v_d+1j*v_q)*np.sqrt(3/2)
            return v_m

        if v_type == 'dq_cplx':
            return v_q+1j*v_d

    if dq_name == 'ri':

        v_r_name = f'v_{bus_name}_{phase_name}_r'
        v_i_name = f'v_{bus_name}_{phase_name}_i'

        v_r = syst.get_value(v_r_name)
        v_i = syst.get_value(v_i_name)

        v_m = 0.0
        if v_type == 'rms_phph':
            v_m = np.abs(v_r+1j*v_i)*np.sqrt(3/2)
            return v_m

        if v_type == 'phasor':
            return v_r+1j*v_i


    
def get_i(syst,bus_name,phase_name='',i_type='rms',dq_name='DQ'):

    if dq_name == 'DQ':
        D_ = 'D'
        Q_ = 'Q'    

        i_d_name = f'i_{bus_name}_{D_}'
        i_q_name = f'i_{bus_name}_{Q_}'

        i_d = syst.get_value(i_d_name)
        i_q = syst.get_value(i_q_name)

        i_m = 0.0
        if i_type == 'rms_phph':
            i_m = np.abs(i_d+1j*i_q)*np.sqrt(1/2)
            return i_m

        if i_type == 'dq_cplx':
            return i_q+1j*i_d

    if dq_name == 'ri':

        i_r_name = f'i_{bus_name}_{phase_name}_r'
        i_i_name = f'i_{bus_name}_{phase_name}_i'

        i_r = syst.get_value(i_r_name)
        i_i = syst.get_value(i_i_name)

        i_m = 0.0
        if i_type == 'rms_phph':
            v_m = np.abs(i_r+1j*i_i)*np.sqrt(3/2)
            return v_m

        if i_type == 'phasor':
            return i_r+1j*i_i

    if dq_name == 'r':

        i_r_name = f'i_{bus_name}_{phase_name}_r'
        i_r = syst.get_value(i_r_name)

        if i_type == 'phasor':
            return i_r
        
        
def get_s(syst,bus_name,s_type='cplx'):
    v_dq = get_v(syst,bus_name,v_type='dq_cplx')
    i_dq = get_i(syst,bus_name,i_type='dq_cplx')

    if s_type == 'cplx':
        s = 3/2*v_dq*(i_dq)
        return s


if __name__ == "__main__":
    
    class sys_class():
        
        def __init__(self): 
     
            self.A = np.array([[ 0.00000000e+00,  3.14159265e+02,  0.00000000e+00,
                 0.00000000e+00,  0.00000000e+00],
               [-4.43027144e-01, -1.42857143e-01, -1.44867645e-01,
                 1.69999669e-01,  0.00000000e+00],
               [-1.80455968e-01,  0.00000000e+00, -6.64265905e-01,
                -2.31113959e-03, -2.50000000e+01],
               [ 1.32845727e+00,  0.00000000e+00,  1.35913375e-02,
                -2.58565604e+00,  0.00000000e+00],
               [-3.90435895e-01,  0.00000000e+00,  2.69427382e+00,
                 4.81098835e-01, -2.00000000e+01]])
            
    sys = sys_class()
    print(damp_report(sys))
    
    

    


import subprocess
import os



from_dir = os.path.join('.','svg')
to_dir = os.path.join('.','png')

for filenameExtension in os.listdir(from_dir):
    print(filenameExtension)
    filename, ext=os.path.splitext(filenameExtension)
    p=subprocess.call(['/usr/bin/inkscape', from_dir + '/' + filename + '.svg', '--export-dpi=300', '--export-png',to_dir + '/' + filename + '.png'])






@numba.njit(cache=True)
def Piecewise(arg):
    out = arg[0][1]
    N = len(arg)
    for it in range(N-1,-1,-1):
        if arg[it][1]: out = arg[it][0]
    return out

@numba.njit(cache=True)
def ITE(arg):
    out = arg[0][1]
    N = len(arg)
    for it in range(N-1,-1,-1):
        if arg[it][1]: out = arg[it][0]
    return out


@numba.njit(cache=True)
def Abs(x):
    return np.abs(x)


@numba.njit(cache=True)
def ini_dae_jacobian_numba(struct,x):
    N_x = struct[0].N_x
    N_y = struct[0].N_y
    struct[0].x[:,0] = x[0:N_x]
    struct[0].y_ini[:,0] = x[N_x:(N_x+N_y)]

    ini(struct,10)
    ini(struct,11) 

    for row,col in zip(struct[0].Fx_ini_rows,struct[0].Fx_ini_cols):
        struct[0].Ac_ini[row,col] = struct[0].Fx_ini[row,col]
    for row,col in zip(struct[0].Fy_ini_rows,struct[0].Fy_ini_cols):
        struct[0].Ac_ini[row,col+N_x] = struct[0].Fy_ini[row,col]
    for row,col in zip(struct[0].Gx_ini_rows,struct[0].Gx_ini_cols):
        struct[0].Ac_ini[row+N_x,col] = struct[0].Gx_ini[row,col]
    for row,col in zip(struct[0].Gy_ini_rows,struct[0].Gy_ini_cols):
        struct[0].Ac_ini[row+N_x,col+N_x] = struct[0].Gy_ini[row,col]
        

@numba.njit(cache=True)
def ini_dae_problem(struct,x):
    N_x = struct[0].N_x
    N_y = struct[0].N_y
    struct[0].x[:,0] = x[0:N_x]
    struct[0].y_ini[:,0] = x[N_x:(N_x+N_y)]

    ini(struct,2)
    ini(struct,3) 
    struct[0].fg[:N_x,:] = struct[0].f[:]
    struct[0].fg[N_x:,:] = struct[0].g[:]    
        
@numba.njit(cache=True)
def ssate(struct,xy):
    for it in range(100):
        ini_dae_jacobian_numba(struct,xy[:,0])
        ini_dae_problem(struct,xy[:,0])
        xy[:] += np.linalg.solve(struct[0].Ac_ini,-struct[0].fg)
        if np.max(np.abs(struct[0].fg[:,0]))<1e-8: break
    N_x = struct[0].N_x
    struct[0].x[:,0] = xy[:N_x,0]
    struct[0].y_ini[:,0] = xy[N_x:,0]
    return xy,it


import sympy as sym
import numpy as np

def acdc_3ph_4w_vdc_q(grid,vsc_data):
    '''
    Converter type v_dc,q_ac 3 phase 4 wire
    
    '''
    
    bus_ac_name = vsc_data['bus_ac']
    bus_dc_name = vsc_data['bus_dc']  
    
    A_value  = vsc_data['A']   
    B_value  = vsc_data['B']   
    C_value  = vsc_data['C']   
    
    ### AC-side
    q_a,q_b,q_c = sym.symbols(f'q_vsc_a_{bus_ac_name},q_vsc_b_{bus_ac_name},q_vsc_c_{bus_ac_name}',real=True)

    p_ac,q_ac,p_dc,p_loss = sym.symbols(f'p_vsc_{bus_ac_name},q_vsc_{bus_ac_name},p_vsc_{bus_dc_name},p_vsc_loss_{bus_ac_name}',real=True)
    p_a_d,p_b_d,p_c_d,p_n_d = sym.symbols(f'p_a_d_{bus_ac_name},p_b_d_{bus_ac_name},p_c_d_{bus_ac_name},p_n_d_{bus_ac_name}',real=True)
    C_a,C_b,C_c = sym.symbols(f'C_a_{bus_ac_name},C_b_{bus_ac_name},C_c_{bus_ac_name}',real=True)
   
    #### AC voltages:
    v_a_r,v_b_r,v_c_r,v_n_r = sym.symbols(f'V_{bus_ac_name}_0_r,V_{bus_ac_name}_1_r,V_{bus_ac_name}_2_r,V_{bus_ac_name}_3_r', real=True)
    v_a_i,v_b_i,v_c_i,v_n_i = sym.symbols(f'V_{bus_ac_name}_0_i,V_{bus_ac_name}_1_i,V_{bus_ac_name}_2_i,V_{bus_ac_name}_3_i', real=True)
    
    #### AC currents:
    i_a_r,i_a_i = sym.symbols(f'i_vsc_{bus_ac_name}_a_r,i_vsc_{bus_ac_name}_a_i',real=True)
    i_b_r,i_b_i = sym.symbols(f'i_vsc_{bus_ac_name}_b_r,i_vsc_{bus_ac_name}_b_i',real=True)
    i_c_r,i_c_i = sym.symbols(f'i_vsc_{bus_ac_name}_c_r,i_vsc_{bus_ac_name}_c_i',real=True)
    i_n_r,i_n_i = sym.symbols(f'i_vsc_{bus_ac_name}_n_r,i_vsc_{bus_ac_name}_n_i',real=True)
    
    #### DC voltages:
    v_pos,v_neg = sym.symbols(f'V_{bus_dc_name}_0_r,V_{bus_dc_name}_1_r', real=True)
    v_posi,v_negi = sym.symbols(f'V_{bus_dc_name}_0_i,V_{bus_dc_name}_1_i', real=True)

    # algebraic states dc side
    i_pos,i_neg,v_og = sym.symbols(f'i_vsc_pos_{bus_dc_name}_sp,i_vsc_{bus_dc_name}_sn, v_og_{bus_dc_name}', real=True)

    v_dc_ref  = sym.Symbol(f'v_dc_{bus_dc_name}_ref', real = True) 

    v_dc = v_dc_ref


    A_loss,B_loss,C_loss = sym.symbols(f'A_{bus_ac_name},B_{bus_ac_name},C_{bus_ac_name}',real=True)
    R_dc,K_dc,R_gdc = sym.symbols(f'R_dc_{bus_dc_name},K_dc_{bus_dc_name},R_gdc_{bus_dc_name}',real=True)
     

    i_a_rms = sym.sqrt(i_a_r**2+i_a_i**2 + 0.001) 
    i_b_rms = sym.sqrt(i_b_r**2+i_b_i**2 + 0.001) 
    i_c_rms = sym.sqrt(i_c_r**2+i_c_i**2 + 0.001) 
    i_n_rms = sym.sqrt(i_n_r**2+i_n_i**2 + 0.001) 

    p_loss_a = A_loss*i_a_rms*i_a_rms + B_loss*i_a_rms + C_loss
    p_loss_b = A_loss*i_b_rms*i_b_rms + B_loss*i_b_rms + C_loss
    p_loss_c = A_loss*i_c_rms*i_c_rms + B_loss*i_c_rms + C_loss
    p_loss_n = A_loss*i_n_rms*i_n_rms + B_loss*i_n_rms + C_loss

    v_a = v_a_r + 1j*v_a_i
    v_b = v_b_r + 1j*v_b_i
    v_c = v_c_r + 1j*v_c_i
    v_n = v_n_r + 1j*v_n_i

    i_a = i_a_r + 1j*i_a_i
    i_b = i_b_r + 1j*i_b_i
    i_c = i_c_r + 1j*i_c_i
    i_n = i_n_r + 1j*i_n_i

    s_a = (v_a - v_n) * sym.conjugate(i_a)
    s_b = (v_b - v_n) * sym.conjugate(i_b)
    s_c = (v_c - v_n) * sym.conjugate(i_c)
    s_n = (v_n) * sym.conjugate(i_n)   
    
    eq_p_a_d =  C_a*p_dc - p_a_d 
    eq_p_b_d =  C_b*p_dc - p_b_d
    eq_p_c_d =  C_c*p_dc - p_c_d
    eq_p_n_d =  sym.re(s_n) - p_n_d
    
    eq_i_a_r =  sym.re(s_a) - p_a_d + p_loss_a + p_loss_n/3
    eq_i_b_r =  sym.re(s_b) - p_b_d + p_loss_b + p_loss_n/3
    eq_i_c_r =  sym.re(s_c) - p_c_d + p_loss_c + p_loss_n/3
    eq_i_a_i =  sym.im(s_a) - q_a
    eq_i_b_i =  sym.im(s_b) - q_b
    eq_i_c_i =  sym.im(s_c) - q_c
    
    eq_i_n_r = i_n_r + i_a_r + i_b_r + i_c_r
    eq_i_n_i = i_n_i + i_a_i + i_b_i + i_c_i
   
    
    eq_p_dc = -p_dc - (v_dc/2*i_pos   - v_dc/2*i_neg)


    v_tp = v_dc/2.0
    v_tn = v_dc/2.0

    eq_i_pos = v_og + v_tp - R_dc*i_pos - v_pos
    eq_i_neg = v_og - v_tn - R_dc*i_neg - v_neg
    eq_v_og = -v_og/R_gdc - i_pos - i_neg 


    grid.dae['g'] += [eq_p_a_d,
                      eq_p_b_d,
                      eq_p_c_d,
                      eq_p_n_d,
                      eq_i_a_r,eq_i_a_i,
                      eq_i_b_r,eq_i_b_i,
                      eq_i_c_r,eq_i_c_i,
                      eq_i_n_r,eq_i_n_i,
                      eq_i_pos,eq_i_neg,eq_v_og,
                      eq_p_dc
                      ]
    
    grid.dae['y_ini'] += [p_a_d,
                      p_b_d,
                      p_c_d,
                      p_n_d,
                      i_a_r,   i_a_i,
                      i_b_r,   i_b_i,
                      i_c_r,   i_c_i,
                      i_n_r,   i_n_i,
                      i_pos,i_neg,v_og,
                      p_dc]

    grid.dae['y_run'] += [p_a_d,
                      p_b_d,
                      p_c_d,
                      p_n_d,
                      i_a_r,   i_a_i,
                      i_b_r,   i_b_i,
                      i_c_r,   i_c_i,
                      i_n_r,   i_n_i,
                      i_pos,i_neg,v_og,
                      p_dc]

    # current injections ac side
    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{bus_ac_name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{bus_ac_name}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(bus_ac_name,ph)
        grid.dae['g'] [idx_r] += -i_s_r
        grid.dae['g'] [idx_i] += -i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        grid.dae['h_dict'].update({f'i_vsc_{bus_ac_name}_{ph}_m':i_s_m})

    # current injections dc side
    idx_r,idx_i = grid.node2idx(f'{bus_dc_name}','a')
    grid.dae['g'] [idx_r] += -i_pos    
    grid.dae['g'] [idx_i] += v_posi/1e3

    idx_r,idx_i = grid.node2idx(f'{bus_dc_name}','b')
    grid.dae['g'] [idx_r] += -i_neg  
    grid.dae['g'] [idx_i] += v_negi/1e3  

    
      
    grid.dae['u_ini_dict'].update({f'v_dc_{bus_dc_name}_ref':800.0}) 
    grid.dae['u_ini_dict'].update({f'{str(q_a)}':0.0,f'{str(q_b)}':0.0,f'{str(q_c)}':0.0}) 

    grid.dae['u_run_dict'].update({f'v_dc_{bus_dc_name}_ref':800.0}) 
    grid.dae['u_run_dict'].update({f'{str(q_a)}':0.0,f'{str(q_b)}':0.0,f'{str(q_c)}':0.0}) 


    #grid.dae['u'].pop(str(v_dc_a_r))
    grid.dae['params_dict'].update({f'A_{bus_ac_name}':A_value,f'B_{bus_ac_name}':B_value,f'C_{bus_ac_name}':C_value})
    grid.dae['params_dict'].update({f'C_a_{bus_ac_name}':1/3,f'C_b_{bus_ac_name}':1/3,f'C_c_{bus_ac_name}':1/3})
    grid.dae['params_dict'].update({f'R_dc_{bus_dc_name}':1e-6})
    grid.dae['params_dict'].update({f'K_dc_{bus_dc_name}':1e-6})
    grid.dae['params_dict'].update({f'R_gdc_{bus_dc_name}':3.0})    

    grid.dae['xy_0_dict'].update({f'v_{bus_dc_name}_a_r':800.0,f'v_{bus_dc_name}_n_r':1.0})
    
    grid.dae['h_dict'].update({f'p_vsc_{bus_ac_name}':sym.re(s_a)+sym.re(s_b)+sym.re(s_c)+sym.re(s_n)})
    grid.dae['h_dict'].update({f'p_vsc_loss_{bus_ac_name}':(p_loss_a+p_loss_b+p_loss_c+p_loss_n)})

def test():
    import numpy as np
    import sympy as sym
    import json
    from pydae.urisi.urisi_builder import urisi
    import pydae.build_cffi as db

    grid = urisi('acdc_3ph_4w_vdc_q.hjson')
    grid.uz_jacs = True
    grid.construct('temp')
    grid.compile('temp')

    import temp

    S_n = 100e3
    V_n = 400
    I_n = S_n/V_n
    Conduction_losses = 0.02*S_n # = A*I_n**2
    lossses = 0.0
    A = Conduction_losses/(I_n**2)/3*lossses
    B = 1/3*lossses
    C = 0.02*S_n/3*lossses
    model = temp.model()
    model.ini({'A_A1':A,'B_A1':B,'C_A1':C},'xy_0.json')
    model.report_y()
    model.report_z()
    #model.save_xy_0('xy_1.json')
    print(800**2/10)



if __name__ == '__main__':

    #development()
    test()



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

from pydae.bmapu.pvs.pv_1 import pv_1
from pydae.bmapu.pvs.pv_dq import pv_dq
from pydae.bmapu.pvs.pv_dq_d import pv_dq_d

def add_pvs(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    for item in grid.data['pvs']:

        data_dict = item
        
        bus_name = item['bus']
        
        if 'name' in item:
            name = item['name']
        else:
            name = bus_name
            
        for gen_id in range(100):
            if name not in grid.generators_id_list:
                grid.generators_id_list += [name]
                break
            else:
                name = name + f'_{gen_id}'
                
        item['name'] = name

        if item['type'] == 'pv_1':                    
            p_W, q_var = pv_1(grid,name,bus_name,data_dict)

        if item['type'] == 'pv_dq':                    
            p_W, q_var = pv_dq(grid,name,bus_name,data_dict)

        if item['type'] == 'pv_dq_d':                    
            p_W, q_var = pv_dq_d(grid,name,bus_name,data_dict)

        # grid power injection
        idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
        if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
        buses[idx_bus]['idx_powers'] += 1

        S_base = sym.Symbol('S_base', real = True)
        grid.dae['g'][idx_bus*2]   += -p_W/S_base
        grid.dae['g'][idx_bus*2+1] += -q_var/S_base


        ## Add controlers and models
        m = f'm_{name}'
        theta_t = f'theta_t_{name}'
        p_s_ref = f'p_s_ref_{name}'
        



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.urisi.pvs.pv_mpt_dcdc import pv_mpt_dcdc
from pydae.urisi.pvs.pv_mpt_dcac import pv_mpt_dcac
from pydae.urisi.pvs.pv_mpt import pv_mpt

def add_pv(grid,item,name,bus_name):

    if item['type'] == 'pv_mpt_dcac':
        pv_mpt_dcac(grid,item,name,bus_name)
    if item['type'] == 'pv_mpt_dcdc':
        pv_mpt_dcdc(grid,item,name,bus_name)
    if item['type'] == 'pv_mpt':
        pv_mpt(grid,item,name,bus_name)
                


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym



def load_ac(grid,data):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    self = grid

    name = data['bus']
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_og_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_o_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_og_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_o_i', real=True)
    K_abc,V_th = sym.symbols(f'K_abc_{name},V_th_{name}',real=True)
    # i_a = I_node_sym_list[nodes_list.index(f'{bus_name}.1')]
    # i_b = I_node_sym_list[nodes_list.index(f'{bus_name}.2')]
    # i_c = I_node_sym_list[nodes_list.index(f'{bus_name}.3')]
    # i_n = I_node_sym_list[nodes_list.index(f'{bus_name}.4')]

    i_a_r,i_a_i = sym.symbols(f'i_load_{name}_a_r,i_load_{name}_a_i', real=True)
    i_b_r,i_b_i = sym.symbols(f'i_load_{name}_b_r,i_load_{name}_b_i', real=True)
    i_c_r,i_c_i = sym.symbols(f'i_load_{name}_c_r,i_load_{name}_c_i', real=True)
    i_n_r,i_n_i = sym.symbols(f'i_load_{name}_n_r,i_load_{name}_n_i', real=True)
    
    i_a = i_a_r + sym.I*i_a_i
    i_b = i_b_r + sym.I*i_b_i
    i_c = i_c_r + sym.I*i_c_i
    i_n = i_n_r + sym.I*i_n_i
    
    v_a = v_sa_r +  sym.I*v_sa_i
    v_b = v_sb_r +  sym.I*v_sb_i
    v_c = v_sc_r +  sym.I*v_sc_i
    v_n = v_sn_r +  sym.I*v_sn_i

    v_an = v_a - v_n
    v_bn = v_b - v_n
    v_cn = v_c - v_n

    v_anm = (v_sa_r**2 + v_sa_i**2)**0.5 
    v_bnm = (v_sb_r**2 + v_sb_i**2)**0.5 
    v_cnm = (v_sc_r**2 + v_sc_i**2)**0.5 

    V_th = 0.7

    Kv_anm_lim = sym.Piecewise(((v_anm+0.3),v_anm<V_th),(1.0,v_anm>=V_th))
    Kv_bnm_lim = sym.Piecewise(((v_bnm+0.3),v_bnm<V_th),(1.0,v_bnm>=V_th))
    Kv_cnm_lim = sym.Piecewise(((v_cnm+0.3),v_cnm<V_th),(1.0,v_cnm>=V_th))

    s_a = v_an*sym.conjugate(i_a)
    s_b = v_bn*sym.conjugate(i_b)
    s_c = v_cn*sym.conjugate(i_c)
    #s = s_a + s_b + s_c

    p_a,p_b,p_c = sym.symbols(f'p_load_{name}_a,p_load_{name}_b,p_load_{name}_c', real=True)
    q_a,q_b,q_c = sym.symbols(f'q_load_{name}_a,q_load_{name}_b,q_load_{name}_c', real=True)
    g_a,g_b,g_c = sym.symbols(f'g_load_{name}_a,g_load_{name}_b,g_load_{name}_c', real=True)
    b_a,b_b,b_c = sym.symbols(f'b_load_{name}_a,b_load_{name}_b,b_load_{name}_c', real=True)

    s_z_a = sym.conjugate((g_a + 1j*b_a)*v_an)*v_an
    s_z_b = sym.conjugate((g_b + 1j*b_b)*v_bn)*v_bn
    s_z_c = sym.conjugate((g_c + 1j*b_c)*v_cn)*v_cn
    
    self.dae['g'] += [K_abc*(p_a + sym.re(s_z_a) + sym.re(s_a)/Kv_anm_lim)]
    self.dae['g'] += [K_abc*(p_b + sym.re(s_z_b) + sym.re(s_b)/Kv_bnm_lim)]
    self.dae['g'] += [K_abc*(p_c + sym.re(s_z_c) + sym.re(s_c)/Kv_cnm_lim)]
    self.dae['g'] += [K_abc*(q_a + sym.im(s_z_a) + sym.im(s_a)/Kv_anm_lim)]
    self.dae['g'] += [K_abc*(q_b + sym.im(s_z_b) + sym.im(s_b)/Kv_bnm_lim)]
    self.dae['g'] += [K_abc*(q_c + sym.im(s_z_c) + sym.im(s_c)/Kv_cnm_lim)]

    self.dae['g'] += [sym.re(i_a+i_b+i_c+i_n)]
    self.dae['g'] += [sym.im(i_a+i_b+i_c+i_n)]

    self.dae['y_ini'] += [i_a_r]
    self.dae['y_ini'] += [i_a_i]
    self.dae['y_ini'] += [i_b_r]
    self.dae['y_ini'] += [i_b_i]
    self.dae['y_ini'] += [i_c_r]
    self.dae['y_ini'] += [i_c_i]
    self.dae['y_ini'] += [i_n_r]
    self.dae['y_ini'] += [i_n_i]

    self.dae['y_run'] += [i_a_r]
    self.dae['y_run'] += [i_a_i]
    self.dae['y_run'] += [i_b_r]
    self.dae['y_run'] += [i_b_i]
    self.dae['y_run'] += [i_c_r]
    self.dae['y_run'] += [i_c_i]
    self.dae['y_run'] += [i_n_r]
    self.dae['y_run'] += [i_n_i]


    for ph in ['a','b','c','n']:
        i_s_r,i_s_i = sym.symbols(f'i_load_{name}_{ph}_r,i_load_{name}_{ph}_i', real=True)
        idx_r,idx_i = self.node2idx(name,ph)
        self.dae['g'] [idx_r] += -i_s_r
        self.dae['g'] [idx_i] += -i_s_i


    p_load_N,q_load_N = 0.0,0.0
    if 'kVA' in data:
        S_N = data['kVA']*1000
        pf_N = data['pf']
        p_load_N = S_N*np.abs(pf_N)
        q_load_N = np.sqrt(S_N**2 - p_load_N**2)*np.sign(pf_N)
    if 'kW' in data:
        p_load_N =  data['kW']*1000
        q_load_N =  data['kvar']*1000

    for phase in ['a','b','c']:
        self.dae['u_ini_dict'].update({f'p_load_{name}_{phase}':p_load_N/3})
        self.dae['u_ini_dict'].update({f'q_load_{name}_{phase}':q_load_N/3})
        self.dae['u_run_dict'].update({f'p_load_{name}_{phase}':p_load_N/3})
        self.dae['u_run_dict'].update({f'q_load_{name}_{phase}':q_load_N/3})
        self.dae['u_ini_dict'].update({f'g_load_{name}_{phase}':0.0})
        self.dae['u_ini_dict'].update({f'b_load_{name}_{phase}':0.0})
        self.dae['u_run_dict'].update({f'g_load_{name}_{phase}':0.0})
        self.dae['u_run_dict'].update({f'b_load_{name}_{phase}':0.0})

    self.dae['params_dict'].update({f'K_abc_{name}':1.0})
    self.dae['h_dict'].update({f'v_anm_{name}':v_anm})
    self.dae['h_dict'].update({f'v_bnm_{name}':v_bnm})
    self.dae['h_dict'].update({f'v_cnm_{name}':v_cnm})


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def load_zip(grid,name,bus_name,data_dict):
    """
    # auxiliar
    
    """

    V = sym.Symbol(f"V_{bus_name}", real=True)
    S_base = sym.Symbol(f"S_base", real=True)    

    T_pz = sym.Symbol(f"T_pz_{name}", real=True)
    T_qz = sym.Symbol(f"T_qz_{name}", real=True)
    
    # inputs and algebraic states
    p_z_f = sym.Symbol(f"p_z_f_{name}", real=True)
    q_z_f = sym.Symbol(f"q_z_f_{name}", real=True)
    p_z = sym.Symbol(f"p_z_{name}", real=True)
    q_z = sym.Symbol(f"q_z_{name}", real=True)
    p_i = sym.Symbol(f"p_i_{name}", real=True)
    q_i = sym.Symbol(f"q_i_{name}", real=True)
    p_p = sym.Symbol(f"p_p_{name}", real=True)
    q_p = sym.Symbol(f"q_p_{name}", real=True)

    i_p = sym.Symbol(f"i_p_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)
    g_load = sym.Symbol(f"g_load_{name}", real=True)
    b_load = sym.Symbol(f"b_load_{name}", real=True)

    # parameters
    
    # auxiliar

    # dynamic equations            
    dp_z_f = 1.0/T_pz*(p_z - p_z_f)
    dq_z_f = 1.0/T_qz*(q_z - q_z_f)

    # algebraic equations 
    eq_p_z = -p_z + g_load*V**2
    eq_q_z = -q_z + b_load*V**2
    eq_p_i = -p_i + i_p*V
    eq_q_i = -q_i + i_q*V 


    # dae 
    grid.dae['f'] += [dp_z_f,dq_z_f]
    grid.dae['x'] += [ p_z_f, q_z_f]
    grid.dae['g'] += [eq_p_i,eq_q_i,eq_p_z,eq_q_z]
    grid.dae['y_ini'] += [i_p,i_q,g_load,b_load]  
    grid.dae['y_run'] += [p_i,q_i,p_z,q_z]  

    K_zp_N,K_zq_N = 0.0,0.0
    K_ip_N,K_iq_N = 0.0,0.0

    if 'K_zp' in data_dict: K_zp_N = data_dict['K_zp']
    if 'K_zq' in data_dict: K_zq_N = data_dict['K_zq']
    if 'K_ip' in data_dict: K_ip_N = data_dict['K_ip']
    if 'K_iq' in data_dict: K_iq_N = data_dict['K_iq']

    K_pp_N = 1.0 - K_zp_N - K_ip_N
    K_pq_N = 1.0 - K_zq_N - K_iq_N

    S_base_N = grid.system['S_base']
    p_z_0 = K_zp_N*data_dict['p_mw']*1e6/S_base_N
    q_z_0 = K_zp_N*data_dict['q_mvar']*1e6/S_base_N
    p_i_0 = K_ip_N*data_dict['p_mw']*1e6/S_base_N
    q_i_0 = K_iq_N*data_dict['q_mvar']*1e6/S_base_N
    p_p_0 = K_pp_N*data_dict['p_mw']*1e6/S_base_N
    q_p_0 = K_pq_N*data_dict['q_mvar']*1e6/S_base_N

    grid.dae['u_ini_dict'].update({f'{str(p_z)}':p_z_0})
    grid.dae['u_ini_dict'].update({f'{str(q_z)}':q_z_0})
    grid.dae['u_ini_dict'].update({f'{str(p_i)}':p_i_0})
    grid.dae['u_ini_dict'].update({f'{str(q_i)}':q_i_0})    
    grid.dae['u_ini_dict'].update({f'{str(p_p)}':p_p_0})
    grid.dae['u_ini_dict'].update({f'{str(q_p)}':q_p_0})    
  
    grid.dae['u_run_dict'].update({f'{str(g_load)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(b_load)}':0.0})            
    grid.dae['u_run_dict'].update({f'{str(i_p)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(i_q)}':0.0})     
    grid.dae['u_run_dict'].update({f'{str(p_p)}':p_p_0})
    grid.dae['u_run_dict'].update({f'{str(q_p)}':q_p_0})  

    grid.dae['xy_0_dict'].update({str(p_z_f):p_z_0})
    grid.dae['xy_0_dict'].update({str(q_z_f):q_z_0})
    grid.dae['xy_0_dict'].update({str(g_load):p_z_0})
    grid.dae['xy_0_dict'].update({str(b_load):q_z_0})
    grid.dae['xy_0_dict'].update({str(i_p):p_i_0})
    grid.dae['xy_0_dict'].update({str(i_q):q_i_0})

    grid.dae['params_dict'].update({str(T_pz):0.1})  
    grid.dae['params_dict'].update({str(T_qz):0.1})  

    p_W   = (-p_z_f - p_i - p_p)*S_base
    q_var = (-q_z_f - q_i - q_p)*S_base

    # outputs
    grid.dae['h_dict'].update({f'p_load_{name}':p_W})     
    grid.dae['h_dict'].update({f'q_load_{name}':q_var})     

    return p_W,q_var


def test():
    import numpy as np
    import sympy as sym
    import hjson
    from pydae.bmapu.bmapu_builder import bmapu
    import pydae.build_cffi as db
    import pytest

    grid = bmapu('zip.hjson')
    grid.checker()
    grid.uz_jacs = True
    grid.verbose = False
    grid.build('temp')

    import temp

    model = temp.model()

    model.ini({},'xy_0.json')
    
    model.report_u()
    model.report_y()
    model.report_z()

    # assert model.get_value('V_1') == pytest.approx(v_ref_1, rel=0.001)
    # # assert model.get_value('q_A2') == pytest.approx(-q_ref, rel=0.05)

    # model.ini({'p_m_1':0.5,'v_ref_1':1.0},'xy_0.json')
    # model.run(1.0,{})
    # model.run(15.0,{'v_ref_1':1.05})
    # model.post()

    # import matplotlib.pyplot as plt

    # fig,axes = plt.subplots()
    # axes.plot(model.Time,model.get_values('V_1'))
    # fig.savefig('ntsst1_step.svg')


if __name__ == '__main__':

    #development()
    test()


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

from pydae.bmapu.syns.milano2ord import milano2ord
from pydae.bmapu.syns.milano4ord import milano4ord
from pydae.bmapu.syns.milano6ord import pai6

from pydae.bmapu.avrs.avrs import add_avr
from pydae.bmapu.govs.govs import add_gov
from pydae.bmapu.psss.psss import add_pss


def add_syns(grid):

    buses = grid.data['buses']
    buses_list = [bus['name'] for bus in buses]

    for item in grid.data['syns']:

        data_dict = item
        
        bus_name = item['bus']
        
        if 'name' in item:
            name = item['name']
        else:
            name = bus_name
            
        for gen_id in range(100):
            if name not in grid.generators_id_list:
                grid.generators_id_list += [name]
                break
            else:
                name = name + f'_{gen_id}'
                
        item['name'] = name
                            
        if 'type' in item:
            if item['type'] == 'pai6':
                p_W, q_var = pai6(grid,name,bus_name,data_dict)
            elif item['type'] == 'milano4':
                p_W, q_var = milano4ord(grid,name,bus_name,data_dict)
            elif item['type'] == 'milano2':
                p_W, q_var = milano2ord(grid,name,bus_name,data_dict)
            else:
                print(f"Synchrnous machine model type {item['type']} not found")
        else:
            p_W, q_var = milano4ord(grid,name,bus_name,data_dict)

        # grid power injection
        idx_bus = buses_list.index(bus_name) # get the number of the bus where the syn is connected
        if not 'idx_powers' in buses[idx_bus]: buses[idx_bus].update({'idx_powers':0})
        buses[idx_bus]['idx_powers'] += 1

        S_base = sym.Symbol('S_base', real = True)
        grid.dae['g'][idx_bus*2]   += -p_W/S_base
        grid.dae['g'][idx_bus*2+1] += -q_var/S_base
        
        v_f = f'v_f_{name}'
        p_m = f'p_m_{name}'
        v_pss = f'v_pss_{name}'
        
        if 'avr' in item:
            add_avr(grid.dae,item,name,bus_name)
            grid.dae['u_ini_dict'].pop(str(v_f))
            grid.dae['u_run_dict'].pop(str(v_f))
            grid.dae['xy_0_dict'].update({str(v_f):1.5})
        if 'gov' in item:
            add_gov(grid.dae,item,name,bus_name)  
            grid.dae['u_ini_dict'].pop(str(p_m))
            grid.dae['u_run_dict'].pop(str(p_m))
            grid.dae['xy_0_dict'].update({str(p_m):0.5})
        if 'pss' in item:
            add_pss(grid.dae,item,name,bus_name)  
            grid.dae['u_ini_dict'].pop(str(v_pss))
            grid.dae['u_run_dict'].pop(str(v_pss))
            grid.dae['xy_0_dict'].update({str(v_pss):0.0})


from pydae.utils import read_data

def load_params(model,data_input):

    data = read_data(data_input)

    if 'syns' in data:
        for syn_data in data['syns']:
            name = syn_data['bus']
            if 'name' in syn_data: name = syn_data['name']
            for item in syn_data:
                if item not in ['avr','pss','gov','bus']:
                    model.set_value(f'{item}_{name}',syn_data[item])
                if item == 'gov':
                    for item_gov in syn_data['gov']:
                        param_name = f'{item_gov}_{name}'
                        if param_name in model.params_list:
                            model.set_value(f'{item_gov}_{name}',syn_data['gov'][item_gov])

    if 'vscs' in data:
        for vsc_data in data['vscs']:
            name = vsc_data['bus']
            if 'name' in vsc_data: name = vsc_data['name']
            for item in vsc_data:
                if item in ['S_n']:
                    model.set_value(f'{item}_{name}',vsc_data[item])

    if 'buses' in data:
        for bus_data in data['buses']:
            name = bus_data['name']

            for item in bus_data:
                if item == 'P_W':
                    model.set_value(f'P_{name}',bus_data[item])





        


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def vsc_l(grid,name,bus_name,data_dict):
    '''

    VSC model with L filter coupling and purely algebraic.
    No control is implemented. 

    parameters
    ----------

    S_n: nominal power in VA
    U_n: nominal rms phase to phase voltage in V
    F_n: nominal frequency in Hz
    X_s: coupling reactance in pu (base machine S_n)
    R_s: coupling resistance in pu (base machine S_n)

    inputs
    ------

    v_dc: dc voltage in pu (when v_dc = 1 and m = 1, v_ac = 1)
    m: modulation index (-)
    theta_t: absolute terminal voltage phase angle (rad)

    example
    -------

    "vscs": [{"type":"vsc_l","S_n":1e6,"U_n":400.0,"F_n":50.0,"X_s":0.1,"R_s":0.01,"monitor":True}]
    
    '''

    sin = sym.sin
    cos = sym.cos

    # inputs
    V_s = sym.Symbol(f"V_{bus_name}", real=True)
    theta_s = sym.Symbol(f"theta_{bus_name}", real=True)
    m = sym.Symbol(f"m_{name}", real=True)
    theta_t = sym.Symbol(f"theta_t_{name}", real=True)
    v_dc = sym.Symbol(f"v_dc_{name}", real=True)
      
    # dynamic states
    #m_f = sym.Symbol(f"m_f_{name}", real=True)

    # algebraic states
    i_si = sym.Symbol(f"i_si_{name}", real=True)
    i_sr = sym.Symbol(f"i_sr_{name}", real=True)       
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)

    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    U_n = sym.Symbol(f"U_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_s = sym.Symbol(f"X_s_{name}", real=True)
    R_s = sym.Symbol(f"R_s_{name}", real=True)
    
    params_list = ['S_n','F_n','U_n','X_s','R_s']
    
    # auxiliar
    v_si = V_s*sin(theta_s)  # v_D, e^(-j)
    v_sr = V_s*cos(theta_s)  # v_Q
    Omega_b = 2*np.pi*F_n
    v_t_m = m*v_dc
    v_tr = v_t_m*cos(theta_t)
    v_ti = v_t_m*sin(theta_t)
    
    # dynamic equations            

    # algebraic equations   
    g_i_si = v_ti - R_s*i_si + X_s*i_sr - v_si  
    g_i_sr = v_tr - R_s*i_sr - X_s*i_si - v_sr 
    g_p_s  = i_si*v_si + i_sr*v_sr - p_s  
    g_q_s  = i_si*v_sr - i_sr*v_si - q_s 

    # dae 
    f_vsg = []
    x_vsg = []
    g_vsg = [g_i_si,g_i_sr,g_p_s,g_q_s]
    y_vsg = [  i_si,  i_sr,  p_s,  q_s]

    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
    grid.dae['u_ini_dict'].update({f'{m}':1.0})
    grid.dae['u_run_dict'].update({f'{m}':1.0})

    grid.dae['u_ini_dict'].update({f'{theta_t}':0.0})
    grid.dae['u_run_dict'].update({f'{theta_t}':0.0})

    grid.dae['u_ini_dict'].update({f'{v_dc}':1.2})
    grid.dae['u_run_dict'].update({f'{v_dc}':1.2})

    grid.dae['xy_0_dict'].update({str(p_s):0.5})
       
    # outputs
    grid.dae['h_dict'].update({f"{str(p_s)}":p_s})
    grid.dae['h_dict'].update({f"{str(q_s)}":q_s})
    grid.dae['h_dict'].update({f"{str(v_ti)}":v_ti})
    grid.dae['h_dict'].update({f"{str(v_tr)}":v_tr})    
    grid.dae['h_dict'].update({f"{str(i_si)}":i_si})
    grid.dae['h_dict'].update({f"{str(i_sr)}":i_sr})

    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    if 'monitor' in data_dict:
        if data_dict['monitor'] == True:
            V_dc_b = U_n*np.sqrt(2)
            grid.dae['h_dict'].update({f"v_dc_v_{name}":v_dc*V_dc_b})
            grid.dae['h_dict'].update({f"v_ac_v_{name}":v_t_m*U_n})
            

    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def ac3ph4wgf(grid,vsc_data):
    '''
    VSC with 3 phase and 4 wire working in open loop as a grid former.

    
    '''

    params_dict  = grid.dae['params_dict']
    u_dict = {}
    h_dict = grid.dae['h_dict']


    alpha = np.exp(2.0/3*np.pi*1j)
    A_0a =  np.array([[1, 1, 1],
                    [1, alpha**2, alpha],
                    [1, alpha, alpha**2]])

    A_a0 = 1/3* np.array([[1, 1, 1],
                        [1, alpha, alpha**2],
                        [1, alpha**2, alpha]])

    omega_coi_i = 0
    HS_coi = 0

    omega_coi = sym.Symbol('omega_coi',real=True)
    xi_freq = sym.Symbol('xi_freq',real=True)
    K_agc = sym.Symbol('K_agc',real=True)

    #vscs = [
    #    {'bus':'B1','S_n':100e3,'R':0.01,'X':0.1,'R_n':0.01,'X_n':0.1,'R_ng':0.01,'X_ng':3.0,'K_f':0.1,'T_f':1.0,'K_sec':0.5,'K_delta':0.001},
    #    ]

    #for vsc in vsc_data:
        
    name = vsc_data['bus']

    # inputs
    e_am_m,e_bm_m,e_cm_m,e_om_m = sym.symbols(f'e_{name}_am_m,e_{name}_bm_m,e_{name}_cm_m,e_{name}_om_m', real=True)
    omega_ref,p_ref = sym.symbols(f'omega_{name}_ref,p_{name}_ref', real=True)
    
    # parameters
    S_n,H,K_f,T_f,K_sec,K_delta  = sym.symbols(f'S_n_{name},H_{name},K_f_{name},T_f_{name},K_sec_{name},K_delta_{name}', real=True)
    R_s,R_sn,R_ng = sym.symbols(f'R_{name}_s,R_{name}_sn,R_{name}_ng', real=True)
    X_s,X_sn,X_ng = sym.symbols(f'X_{name}_s,X_{name}_sn,X_{name}_ng', real=True)
    
    # dynamical states
    phi = sym.Symbol(f'phi_{name}', real=True)
    phi_a = sym.Symbol(f'phi_a_{name}', real=True)
    phi_b = sym.Symbol(f'phi_b_{name}', real=True)
    phi_c = sym.Symbol(f'phi_c_{name}', real=True)

    omega = sym.Symbol(f'omega_{name}', real=True)
    
    # algebraic states
    #e_an_i,e_bn_i,e_cn_i,e_ng_i = sym.symbols(f'e_{name}_an_i,e_{name}_bn_i,e_{name}_cn_i,e_{name}_ng_i', real=True)
    v_sa_r,v_sb_r,v_sc_r,v_sn_r,v_ng_r = sym.symbols(f'V_{name}_0_r,V_{name}_1_r,V_{name}_2_r,V_{name}_3_r,v_{name}_n_r', real=True)
    v_sa_i,v_sb_i,v_sc_i,v_sn_i,v_ng_i = sym.symbols(f'V_{name}_0_i,V_{name}_1_i,V_{name}_2_i,V_{name}_3_i,v_{name}_n_i', real=True)
    i_sa_r,i_sb_r,i_sc_r,i_sn_r,i_ng_r = sym.symbols(f'i_vsc_{name}_a_r,i_vsc_{name}_b_r,i_vsc_{name}_c_r,i_vsc_{name}_n_r,i_vsc_{name}_ng_r', real=True)
    i_sa_i,i_sb_i,i_sc_i,i_sn_i,i_ng_i = sym.symbols(f'i_vsc_{name}_a_i,i_vsc_{name}_b_i,i_vsc_{name}_c_i,i_vsc_{name}_n_i,i_vsc_{name}_ng_i', real=True)
    v_mn_r,v_mn_i = sym.symbols(f'v_{name}_mn_r,v_{name}_mn_i', real=True)

    omega = sym.Symbol(f'omega_{name}', real=True)
    
    e_om_r,e_om_i = sym.symbols(f'e_{name}_om_r,e_{name}_om_i', real=True)
    
    Z_sa = R_s + 1j*X_s
    Z_sb = R_s + 1j*X_s
    Z_sc = R_s + 1j*X_s
    Z_sn = R_sn + 1j*X_sn
    Z_ng = R_ng + 1j*X_ng

    i_sa = i_sa_r + 1j*i_sa_i
    i_sb = i_sb_r + 1j*i_sb_i
    i_sc = i_sc_r + 1j*i_sc_i
    i_sn = i_sn_r + 1j*i_sn_i
    i_ng = i_ng_r + 1j*i_ng_i

    v_sa = v_sa_r + 1j*v_sa_i
    v_sb = v_sb_r + 1j*v_sb_i
    v_sc = v_sc_r + 1j*v_sc_i
    v_sn = v_sn_r + 1j*v_sn_i
    v_ng = v_ng_r + 1j*v_ng_i
    v_mn = v_mn_r + 1j*v_mn_i
    
    e_am_r = e_am_m*sym.cos(phi_a + phi) 
    e_am_i = e_am_m*sym.sin(phi_a + phi) 
    e_bm_r = e_bm_m*sym.cos(phi_b + phi-2/3*np.pi) 
    e_bm_i = e_bm_m*sym.sin(phi_b + phi-2/3*np.pi) 
    e_cm_r = e_cm_m*sym.cos(phi_c + phi-4/3*np.pi) 
    e_cm_i = e_cm_m*sym.sin(phi_c + phi-4/3*np.pi) 
    
    e_am_cplx = e_am_r + 1j*e_am_i
    e_bm_cplx = e_bm_r + 1j*e_bm_i
    e_cm_cplx = e_cm_r + 1j*e_cm_i
    e_om_cplx = e_om_r + 1j*e_om_i

    v_san = v_sa - v_sn
    v_sbn = v_sb - v_sn
    v_scn = v_sc - v_sn

    eq_i_sa_cplx = e_am_cplx - i_sa*Z_sa - v_san - v_mn
    eq_i_sb_cplx = e_bm_cplx - i_sb*Z_sb - v_sbn - v_mn
    eq_i_sc_cplx = e_cm_cplx - i_sc*Z_sc - v_scn - v_mn
    eq_v_nm_cplx = 0*e_om_cplx - i_sn*Z_sn - v_mn
    eq_i_sn_cplx = i_sa + i_sb + i_sc + i_sn
    #eq_i_sn_cplx = e_ng_cplx - i_sn*Z_sn - v_ng
    #eq_i_ng_cplx = i_ng + i_sa + i_sb + i_sc + i_sn
    #eq_e_ng_cplx  = -e_ng_cplx  + i_ng*Z_ng

    grid.dae['g'] += [sym.re(eq_i_sa_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sb_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sc_cplx)] 
    grid.dae['g'] += [sym.re(eq_v_nm_cplx)] 
    grid.dae['g'] += [sym.re(eq_i_sn_cplx)] 
    
    #g_list += [sym.re(eq_i_ng_cplx)] 
    #g_list += [sym.re(eq_e_ng_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sa_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sb_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sc_cplx)] 
    grid.dae['g'] += [sym.im(eq_v_nm_cplx)] 
    grid.dae['g'] += [sym.im(eq_i_sn_cplx)] 
    #g_list += [sym.im(eq_i_ng_cplx)] 
    #g_list += [sym.im(eq_e_ng_cplx)]

    grid.dae['y_ini'] += [i_sa_r,i_sb_r,i_sc_r,v_mn_r,i_sn_r]
    grid.dae['y_ini'] += [i_sa_i,i_sb_i,i_sc_i,v_mn_i,i_sn_i]

    grid.dae['y_run'] += [i_sa_r,i_sb_r,i_sc_r,v_mn_r,i_sn_r]
    grid.dae['y_run'] += [i_sa_i,i_sb_i,i_sc_i,v_mn_i,i_sn_i]


    for ph in ['a','b','c','n']:
        i_s_r = sym.Symbol(f'i_vsc_{name}_{ph}_r', real=True)
        i_s_i = sym.Symbol(f'i_vsc_{name}_{ph}_i', real=True)  
        idx_r,idx_i = grid.node2idx(name,ph)
        grid.dae['g'] [idx_r] += i_s_r
        grid.dae['g'] [idx_i] += i_s_i
        i_s = i_s_r + 1j*i_s_i
        i_s_m = np.abs(i_s)
        h_dict.update({f'i_vsc_{name}_{ph}_m':i_s_m})

        
    V_1 = 400/np.sqrt(3)
    #    V_1 = 400/np.sqrt(3)*np.exp(1j*np.deg2rad(0))
    # A_1toabc = np.array([1, alpha**2, alpha])
    #V_abc = V_1 * A_1toabc 
    #e_an_r,e_bn_r,e_cn_r = V_abc.real
    #e_an_i,e_bn_i,e_cn_i = V_abc.imag

    u_dict.update({f'e_{name}_am_m':V_1,f'e_{name}_bm_m':V_1,f'e_{name}_cm_m':V_1,f'e_{name}_om_m':0.0})
    #u_dict.update({f'phi_{name}':0.0})
    u_dict.update({f'phi_a_{name}':0.0})
    u_dict.update({f'phi_b_{name}':0.0})
    u_dict.update({f'phi_c_{name}':0.0})

    u_dict.update({f'p_{name}_ref':0.0})
    u_dict.update({f'omega_{name}_ref':1.0})

    grid.dae['u_ini_dict'].update(u_dict)
    grid.dae['u_run_dict'].update(u_dict)

    #for ph in ['a','b','c','n']:
    #    u_dict.pop(f'i_{name}_{ph}_r')
    #    u_dict.pop(f'i_{name}_{ph}_i')

    params_dict.update({f'X_{name}_s':vsc_data['X'],f'R_{name}_s':vsc_data['R']})
    params_dict.update({f'X_{name}_sn':vsc_data['X_n'],f'R_{name}_sn':vsc_data['R_n']})
    params_dict.update({f'X_{name}_ng':vsc_data['X_ng'],f'R_{name}_ng':vsc_data['R_ng']})
    
    params_dict.update({f'S_n_{name}':vsc_data['S_n']})

    params_dict.update({f'K_f_{name}':vsc_data['K_f']})
    params_dict.update({f'T_f_{name}':vsc_data['T_f']})
    params_dict.update({f'K_sec_{name}':vsc_data['K_sec']})
    params_dict.update({f'K_delta_{name}':vsc_data['K_delta']})
    
    
    v_sabc = sym.Matrix([[v_sa],[v_sb],[v_sc]])
    i_sabc = sym.Matrix([[i_sa],[i_sb],[i_sc]])
    
    v_szpn = A_a0*v_sabc
    i_szpn = A_a0*i_sabc
    
    s_pos = 3*v_szpn[1]*sym.conjugate(i_szpn[1])
    s_neg = 3*v_szpn[2]*sym.conjugate(i_szpn[2])
    s_zer = 3*v_szpn[0]*sym.conjugate(i_szpn[0])
    
    p_pos = sym.re(s_pos)
    
    dphi   = 2*np.pi*50*(omega - omega_coi) - K_delta*phi
    domega = 1/T_f*(omega_ref + K_f*(p_ref + K_sec*xi_freq - p_pos)/S_n - omega)
    
    grid.dae['f'] += [dphi,domega]
    grid.dae['x'] += [ phi, omega]
    
    h_dict.update({f'p_{name}_pos':sym.re(s_pos),f'p_{name}_neg':sym.re(s_neg),f'p_{name}_zer':sym.re(s_zer)})
    h_dict.update({str(e_am_m):e_am_m,str(e_bm_m):e_bm_m,str(e_cm_m):e_cm_m})
    h_dict.update({str(p_ref):p_ref,str(omega_ref):omega_ref})
    h_dict.update({str(omega):omega})
    HS_coi  = S_n
    omega_coi_i = S_n*omega

    grid.omega_coi_numerator += omega_coi_i
    grid.omega_coi_denominator += HS_coi


# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym
from pydae.bmapu.miscellaneous.fault import add_fault


def add_miscellaneous(grid):

    if 'faults' in grid.data:
        for item in grid.data['faults']:
            add_fault(grid,item)



# -*- coding: utf-8 -*-
"""
Created on Thu August 10 23:52:55 2022

@author: jmmauricio
"""

import numpy as np
import sympy as sym

def genape_inf(grid,name,bus_name,data_dict):
    '''

    parameters
    ----------

    S_n: nominal power in VA
    F_n: nominal frequency in Hz
    X_v: coupling reactance in pu (base machine S_n)
    R_v: coupling resistance in pu (base machine S_n)
    K_delta: if K_delta>0.0 current generator is converted to reference machine 
    K_alpha: alpha gain to obtain Domega integral 

    inputs
    ------

    alpha: RoCoF in pu if K_alpha = 1.0
    omega_ref: frequency in pu
    v_ref: internal voltage reference

    example
    -------

    "genapes": [{"S_n":1e9,"F_n":50.0,"X_v":0.001,"R_v":0.0,"K_delta":0.001,"K_alpha":1e-6}]

    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_v = sym.Symbol(f"X_v_{name}", real=True)
    R_v = sym.Symbol(f"R_v_{name}", real=True)
    K_delta = sym.Symbol(f"K_delta_{name}", real=True)
    K_alpha = sym.Symbol(f"K_alpha_{name}", real=True)

    
    '''

    sin = sym.sin
    cos = sym.cos

    
    # inputs
    V = sym.Symbol(f"V_{bus_name}", real=True)
    theta = sym.Symbol(f"theta_{bus_name}", real=True)
    omega_coi = sym.Symbol("omega_coi", real=True)   
    alpha = sym.Symbol(f"alpha_{name}", real=True)
    v_ref = sym.Symbol(f"v_ref_{name}", real=True)
    omega_ref = sym.Symbol(f"omega_ref_{name}", real=True) 
    phi = sym.Symbol(f"phi_{name}", real=True) 
    delta_ref = sym.Symbol(f"delta_ref_{name}", real=True) 
    rocov = sym.Symbol(f"rocov_{name}", real=True) 
    
    # dynamic states
    delta = sym.Symbol(f"delta_{name}", real=True)
    omega = sym.Symbol(f"omega_{name}", real=True)
    Domega = sym.Symbol(f"Domega_{name}", real=True)
    Dv = sym.Symbol(f"Dv_{name}", real=True)

    # algebraic states
    i_d = sym.Symbol(f"i_d_{name}", real=True)
    i_q = sym.Symbol(f"i_q_{name}", real=True)            
    p_s = sym.Symbol(f"p_s_{name}", real=True)
    q_s = sym.Symbol(f"q_s_{name}", real=True)
    
    # parameters
    S_n = sym.Symbol(f"S_n_{name}", real=True)
    F_n = sym.Symbol(f"F_n_{name}", real=True)            
    X_v = sym.Symbol(f"X_v_{name}", real=True)
    R_v = sym.Symbol(f"R_v_{name}", real=True)
    K_delta = sym.Symbol(f"K_delta_{name}", real=True)
    K_alpha = sym.Symbol(f"K_alpha_{name}", real=True)
    K_rocov = sym.Symbol(f"K_rocov_{name}", real=True)
    
    params_list = ['S_n','F_n','X_v','R_v','K_delta','K_alpha']
    
    # auxiliar
    v_d = V*sin(delta + phi - theta) 
    v_q = V*cos(delta + phi - theta) 
    Omega_b = 2*np.pi*F_n
    omega_s = omega_coi
    e_dv = 0  
    e_qv = v_ref + Dv  
    
    # dynamic equations            
    ddelta = Omega_b*(omega - omega_s) - K_delta*(delta - delta_ref)
    dDomega = alpha - K_alpha*Domega
    dDv = rocov - K_rocov*Dv

    # algebraic equations   
    g_omega = -omega + Domega + omega_ref
    g_i_d  = -R_v*i_d + X_v*i_q - v_d + e_dv 
    g_i_q  = -R_v*i_q - X_v*i_d - v_q + e_qv
    g_p_s  = i_d*v_d + i_q*v_q - p_s  
    g_q_s  = i_d*v_q - i_q*v_d - q_s 
    
    
    # dae 
    f_vsg = [ddelta,dDomega,dDv]
    x_vsg = [ delta, Domega, Dv]
    g_vsg = [g_omega,g_i_d,g_i_q,g_p_s,g_q_s]
    y_vsg = [  omega,  i_d,  i_q,  p_s,  q_s]
    
    H = 1e6
    grid.H_total += H
    grid.omega_coi_numerator += omega*H*S_n
    grid.omega_coi_denominator += H*S_n

    grid.dae['f'] += f_vsg
    grid.dae['x'] += x_vsg
    grid.dae['g'] += g_vsg
    grid.dae['y_ini'] += y_vsg  
    grid.dae['y_run'] += y_vsg  
    
        
    grid.dae['u_ini_dict'].update({f'{str(alpha)}':0})
    grid.dae['u_run_dict'].update({f'{str(alpha)}':0})

    grid.dae['u_ini_dict'].update({f'{str(v_ref)}':1.0})
    grid.dae['u_run_dict'].update({f'{str(v_ref)}':1.0})

    grid.dae['u_ini_dict'].update({f'{str(omega_ref)}':1.0})
    grid.dae['u_run_dict'].update({f'{str(omega_ref)}':1.0})

    grid.dae['u_ini_dict'].update({f'{str(delta_ref)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(delta_ref)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(phi)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(phi)}':0.0})

    grid.dae['u_ini_dict'].update({f'{str(rocov)}':0.0})
    grid.dae['u_run_dict'].update({f'{str(rocov)}':0.0})

    grid.dae['xy_0_dict'].update({str(omega):1.0})
       
    # outputs
    grid.dae['h_dict'].update({f"alpha_{name}":alpha})
    grid.dae['h_dict'].update({f"Dv_{name}":Dv})
    
    for item in params_list:       
        grid.dae['params_dict'].update({f"{item}_{name}":data_dict[item]}) 

    grid.dae['params_dict'].update({f"K_rocov_{name}":1e-6}) 

    p_W   = p_s * S_n
    q_var = q_s * S_n

    return p_W,q_var


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
2020

@author: jmmauricio
"""

import numpy as np
from cycler import cycler
import matplotlib.pyplot as plt

def set_style(plt):

    plt.rcParams['axes.prop_cycle'] = cycler('color', ['#d9524f', '#5cb85c', '#337ab7', '#f0ad4e', '#5bc0de','#5e4485'])
    #cycler('color',['#d9524f', '5cb85c', '337ab7', 'f0ad4e', '5bc0de','5e4485']) # jmcolo 
    colors = plt.rcParams['axes.prop_cycle'].by_key()['color']
    # Set default figure size
    plt.rcParams['figure.figsize'] = (3.3, 2.5)
    #plt.rcParams['figure.dpi'] : 600

    # Font sizes
    plt.rcParams['font.size'] =  10
    plt.rcParams['font.family'] = 'serif'
    plt.rcParams['font.serif'] = 'Times New Roman'
    plt.rcParams['mathtext.fontset'] = 'cm' #'stix'
    #plt.rcParams['text.usetex'] = True
    plt.rcParams['legend.handlelength'] = 1.

    return colors


def plot(model,layout,file_name=''):
    '''
    layout = [[['v_ref_1'],['V_1','V_2']],
              [['omega_1'],['theta_1']]]

    layout = [['v_ref_1'],['V_1','V_2']]

    layout = [[['v_ref_1'],
               ['omega_1']]]

    layout = ['v_ref_1','omega_1']
    
    '''


    
    colors = set_style(plt)
    nrows = len(layout)
    if type(layout[0]) == list:
        ncols = len(layout[0])
        figsize = (3.3*ncols,3*nrows)
        fig,axes = plt.subplots(nrows=nrows,ncols=ncols,figsize=figsize)
    else:
        nrows = 1
        ncols = 0
        figsize = (3.3,3)
        fig,axes = plt.subplots(figsize=figsize)


    fig.tight_layout()
    
    if ncols>1 and nrows>1:
        for irow in range(nrows):
            for icol in range(ncols):
                for item in layout[irow][icol]:
                    axes[irow,icol].plot(model.Time,model.get_values(item),label=item)
                axes[irow,icol].grid()
                axes[irow,icol].legend()
                axes[irow,icol].set_xlim([model.Time[0],model.Time[-1]])
        for icol in range(ncols):
            axes[-1,icol].set_xlabel('Time (s)')
        
    if nrows>1 and ncols==1:
        for irow in range(nrows):
            for item in layout[irow]:
                axes[irow].plot(model.Time,model.get_values(item),label=item)
            axes[irow].grid()
            axes[irow].legend()
            axes[irow].set_xlim([model.Time[0],model.Time[-1]])
            axes[-1].set_xlabel('Time (s)')

    if nrows==1 and ncols>1:
        for icol in range(ncols):
            for item in layout[0][icol]:
                axes[icol].plot(model.Time,model.get_values(item),label=item)
            axes[icol].grid()
            axes[icol].legend()
            axes[icol].set_xlim([model.Time[0],model.Time[-1]])
            axes[icol].set_xlabel('Time (s)')

    if nrows==1 and ncols==0:
        for item in layout:
            axes.plot(model.Time,model.get_values(item),label=item)
        axes.grid()
        axes.legend()
        axes.set_xlim([model.Time[0],model.Time[-1]])
        axes.set_xlabel('Time (s)')

    if file_name != '':
        fig.savefig(file_name)

    return fig

